# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import xbmc  # @UnresolvedImport
import xbmcaddon   # @UnresolvedImport
import xbmcvfs   # @UnresolvedImport
import xbmcgui   # @UnresolvedImport
import sys
import os
import traceback

from threading import Condition
from threading import Thread
from ..utils.Logger import Logger
from ..utils.ImageLoader import ImageLoader
from ..api.LoginException import LoginException
from ..api.API import API
from ..streamer.Streamer import Streamer
from ..api.VersionException import VersionException
from ..constants import const
from ..panels.SplashScreenPanel import SplashScreenPanel


IMG_CACHE="img_cache"
DOWNLOADS="downloads"
EPG="epg"
PICONS="picons"
PROC="proc"
XTREAM="xtream"
DATA_FOLDERS=[IMG_CACHE,DOWNLOADS,EPG,PICONS,PROC,XTREAM]

class Timer(Thread):
    def __init__(self):
        Thread.__init__(self)
        self.setDaemon(True)
        self.callback=[]
        self._cancelled=False
        self.condition=Condition()
        self.timeout=None
        self.one_shot=True        
        pass
    
    def run(self):
        while True:
            self.condition.acquire()
            try:
                if(self._cancelled==False and self.timeout is not None):
                    self.condition.wait(self.timeout)                
                if(self._cancelled==False):
                    for callback in self.callback:
                        try:
                            callback()
                        except Exception:
                            pass
                else:
                    break
                if(self.one_shot==False):
                    continue
                else:
                    break
            finally:
                self.condition.notifyAll()
                self.condition.release()
        
    def stop(self):
        self.condition.acquire()
        self._cancelled=True
        try:
            pass
        finally:
            self.condition.notifyAll()
            self.condition.release()
    
    def start(self, timeout, one_shot=True):
        self.timeout=timeout
        self.one_shot=one_shot
        Thread.start(self)        


class Context(Logger):
    
    def __init__(self):
        Logger.__init__(self)
        
        self.addon = xbmcaddon.Addon(id='plugin.video.seasondream')
        self._native_path = xbmc.translatePath(self.addon.getAddonInfo('path'))
        self._plugin_id=self.addon.getAddonInfo('id')
        self._plugin_name=self.addon.getAddonInfo('name')
        self._plugin_handle = int(sys.argv[1])
        self._data_path = xbmc.translatePath('special://profile/addon_data/%s' % self._plugin_id)
        
        if isinstance(self._data_path, str):
            self._data_path = self._data_path.decode('utf-8')
            
        if not xbmcvfs.exists(self._data_path):
            xbmcvfs.mkdir(self._data_path)
            
        for folder in DATA_FOLDERS:
            if not xbmcvfs.exists(self._data_path+folder):
                xbmcvfs.mkdir(os.path.join(self._data_path,folder))
            self.log("%s path: %s"%(folder,os.path.join(self._data_path,folder)))

        self._splash_screen=SplashScreenPanel(context=self)
        self.streamer=None
        self.timers=[]
        
        self.log(self._data_path)
        self.log(self._plugin_name)
        self.log(self._native_path)
    
    def getTimer(self):
        timer=Timer()
        self.timers.append(timer)
        return timer
    
    def getStreamer(self, url, mode):
        if(self.streamer is not None):
            self.streamer.cancel()
        self.streamer=Streamer(url=url,mode=mode, context=self)
        return self.streamer
    
    def _isRunning(self):
        if(self.getHandle()==1):
            return False
        h=-1
        try:
            with open(os.path.join(self.getProcPath(),'proc'),'rb') as f:
                h=(int)(f.read())
            return h!=-1
        except Exception,e:
            self.log("Unable to get handle from proc %s"%str(e))
            return False

    def _setHandle(self, handle=None):
        if(handle is None):
            handle=self.getHandle()
        try:
            with open(os.path.join(self.getProcPath(),'proc'),'wb') as f:
                f.write(str(handle))
            self.log("Handle %s set"%str(handle))
        except Exception,e:
            self.log("Unable to set handle %s"%str(e))
    
    def shutDown(self):
        self._setHandle(handle=-1)
        self.hideSplashScreen()
        for timer in self.timers:
            timer.stop()
        
    def showSplashScreen(self):
        try:
            if(self._splash_screen is not None):
                self._splash_screen.show()
        except Exception:
            pass
    
    def hideSplashScreen(self):
        try:
            if(self._splash_screen is not None):
                self._splash_screen.close()
                del self._splash_screen
        except Exception:
            pass
    
    def setNoticeText(self, text):
        if(self._splash_screen is not None and text is not None):
            self._splash_screen.setInfoText(text)
    
    def init(self):
        try:
            if(self._isRunning()):
                raise RuntimeError("Instance already running")
            self._setHandle()
            plugin_path=self.createResourcePath('lib')
            logo_path=self.createResourcePath('media')
            on_notice=self.setNoticeText
            if(self.getSetting("username")=="Demo"):
                try:
                    import re
                    r=re.compile(".{2}:.{2}:.{2}:.{2}:.{2}:.{2}")
                    api=API(username="", password="",version_major=0,version_minor=0,noAction=True)
                    mac = xbmc.getInfoLabel('Network.MacAddress')
                    for _ in range(1,20):
                        if(r.match(mac) is not None):
                            self.log("MAC found")
                            break
                        else:
                            xbmc.sleep(500)
                            mac = xbmc.getInfoLabel('Network.MacAddress')
                    account=api.createAccount(mac=mac)
                    self.log("Got new account "+str(account["login"]))
                    self.setSetting("username",str(account["login"]))
                    self.setSetting("password",str(account["password"]))
                    self.log("New account saved")
                                    
                except Exception,e:
                    self.log("Unable to add account. "+str(e))

            # API init
            self.log("Initializing API...")
            try:
                self._api=API(username=self.getSetting("username"), password=self.getSetting("password"),client="3",version_major=const.VERSION_MAJOR,version_minor=const.VERSION_MINOR, plugin_path=plugin_path, logo_path=logo_path, on_notice=on_notice)
                self.log("Susseccful")
            except LoginException, le:
                dialog = xbmcgui.Dialog()
                dialog.ok("Erorr", le.message)
                return False
            except VersionException, ve:
                dialog = xbmcgui.Dialog()
                dialog.ok("Erorr", ve.message)
                return False
            except Exception, e:
                dialog = xbmcgui.Dialog()
                dialog.ok("API Erorr", str(e))
                traceback.print_exc()            
                return False
            
            self._imgLoader=ImageLoader(imagePath=self.getImgCachePath(), cacheSize=(int)(self.getSetting("cache.size")))
            
            self.log("Context has been initialized")
                
            return True
        except RuntimeError,re:
            self.log("Runtime error %s"%str(re))
        except Exception,e:
            self.log("Unable to initialize context %s"%str(e))
            traceback.print_exc()            
        return False
    
    def getImageForMovie(self, movie):
        return self._imgLoader._getDestFile(movie)
    
    def populateImage(self, movie):
        self.getImageLoader().populateImage(movie)
        
    def getImageLoader(self):
        return self._imgLoader
    
    def getApi(self):
        return self._api
    
    def getSetting(self, settingId):
        return self.addon.getSetting(settingId)
    
    def setSetting(self, settingId, value):
        return self.addon.setSetting(settingId, value)
    
    def createResourcePath(self, *args):
        path_comps = []
        for arg in args:
            path_comps.extend(arg.split('/'))
        path = os.path.join(self.getNativePath(), 'resources', *path_comps)
        return path
    
    def getHandle(self):
        return self._plugin_handle
    
    def getNativePath(self):
        return self._native_path
    
    def getDataPath(self):
        return self._data_path
    
    def getImgCachePath(self):
        return os.path.join(self.getDataPath(),IMG_CACHE)

    def getProcPath(self):
        return os.path.join(self.getDataPath(),PROC)
    
    def getDownloadsPath(self):
        return os.path.join(self.getDataPath(),DOWNLOADS)
    
    def getEPGPath(self):
        return os.path.join(self.getDataPath(),EPG)
    
    def getPiconsPath(self):
        return os.path.join(self.getDataPath(),PICONS)

    def getXtreamPath(self):
        return os.path.join(self.getDataPath(),XTREAM)
    def _encode(self, text):
        try:
            return text.encode("utf-8")
        except Exception:
            return text
        
    def localize(self, text_id, default_text=u''):
        if isinstance(text_id, int):
            if text_id >= 0 and (text_id < 30000 or text_id > 30999):
                result = xbmc.getLocalizedString(text_id)
                if result is not None and result:
                    return self._encode(result)  
                pass
            pass
        result = self.addon.getLocalizedString(int(text_id))
        if result is not None and result:
            return self._encode(result)

        return self._encode(default_text)    
