# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime

from ..utils.Logger import Logger
from ..constants import const

import xbmc, xbmcgui  # @UnresolvedImport

class ControlSliderStar:
    IMAGES=['starsbar_empty.png','starsbar_filled_1.png','starsbar_filled_2.png','starsbar_filled_3.png','starsbar_filled_4.png','starsbar_filled_5.png','starsbar_filled_6.png','starsbar_filled_7.png','starsbar_filled_8.png','starsbar_filled_9.png','starsbar_filled_10.png']

    def __init__(self,x,y, context, window):
        self.context=context
        self.background=xbmcgui.ControlImage(x,y,100,20,self.context.createResourcePath('media','starsbar_empty.png'))
        self.foreground=xbmcgui.ControlImage(x,y,100,20,'')
        self.label=xbmcgui.ControlLabel(x+102,y-2,35,20,'',font='font12',alignment=0)        
        self.hide()
        window.addControl(self.background)
        window.addControl(self.foreground)
        window.addControl(self.label)
        
    def setValue(self, value):
        value=(int)(min(max(value,0),10))
        self.foreground.setImage(self.context.createResourcePath('media',self.IMAGES[value]), useCache=True)
        self.label.setLabel("%.1f" % value)

    def show(self):
        self.background.setVisible(True)
        self.foreground.setVisible(True)
        self.label.setVisible(True)
        
    def hide(self):        
        self.background.setVisible(False)
        self.foreground.setVisible(False)
        self.label.setVisible(False)

class ControlClock:
    
    def __init__(self, x,y,context, window,pattern="%H:%M"):
        self.context=context
        self.pattern=pattern
        self.label=xbmcgui.ControlLabel(x,y,100,25,self._getTimeString(),font='font13',alignment=1) #1-right, 2-center
        window.addControl(self.label)
        self.timer=self.context.getTimer()
        self.timer.callback.append(self.onUpdate)
        self.timer.start(1,False)
    
    def cancel(self):
        self.timer.stop()
    
    def _getTimeString(self):
        return datetime.now().strftime(self.pattern)
    
    def onUpdate(self):
        try:
            self.label.setLabel(self._getTimeString())
        except Exception:
            self.timer.stop()
            
        

class BasePanel(xbmcgui.Window,Logger):
    def __init__(self, context=None, background=None):
        Logger.__init__(self)
        self.context=context
        self.pageToken=""
        if(background is not None):
            self.addControl(xbmcgui.ControlImage(0,0,1280,720,self.context.createResourcePath('media',background)))
        
        self.title=xbmcgui.ControlTextBox(10,10,600,25)
        self.addControl(self.title)

        self.version=xbmcgui.ControlLabel(1138,10,130,25,'',font='font13',alignment=1) #1-right, 2-center
        self.addControl(self.version)

        self.buttons=[]
        
        self.playlistButton=xbmcgui.ControlButton(977,682,35,35,u'',self.context.createResourcePath('media','playlist_focused.png'),self.context.createResourcePath('media','playlist.png'))
        self.addControl(self.playlistButton)
        self.buttons.append(self.playlistButton)
        self.playlistButton.setEnabled(False)

        self.tvButton=xbmcgui.ControlButton(1022,682,35,35,u'',self.context.createResourcePath('media','tv_focused.png'),self.context.createResourcePath('media','tv.png'))
        self.addControl(self.tvButton)
        self.buttons.append(self.tvButton)

        self.genreButton=xbmcgui.ControlButton(1065,682,35,35,u'',self.context.createResourcePath('media','genre_focused.png'),self.context.createResourcePath('media','genre.png'))
        self.addControl(self.genreButton)
        self.buttons.append(self.genreButton)

        self.favButton=xbmcgui.ControlButton(1108,682,35,35,u'',self.context.createResourcePath('media','fav_focused.png'),self.context.createResourcePath('media','fav.png'))
        self.addControl(self.favButton)
        self.buttons.append(self.favButton)

        self.searchButton=xbmcgui.ControlButton(1151,682,35,35,u'',self.context.createResourcePath('media','search_focused.png'),self.context.createResourcePath('media','search.png'))
        self.addControl(self.searchButton)
        self.buttons.append(self.searchButton)

        self.prevButton=xbmcgui.ControlButton(1194,682,35,35,u'',self.context.createResourcePath('media','left_focused.png'),self.context.createResourcePath('media','left.png'))
        self.addControl(self.prevButton)
        self.buttons.append(self.prevButton)

        self.nextButton=xbmcgui.ControlButton(1237,682,35,35,u'',self.context.createResourcePath('media','right_focused.png'),self.context.createResourcePath('media','right.png'))
        self.addControl(self.nextButton)
        self.buttons.append(self.nextButton)
        
        for i in range(0,len(self.buttons)):
            self.buttons[i].controlRight(self.buttons[(i+1)%len(self.buttons)])
            self.buttons[i].controlLeft(self.buttons[(i-1+len(self.buttons))%len(self.buttons)])
        
        self.setVersionText(self.getVersion())
    
    def setTitleText(self, text):
        self.title.setText(text)

    def setVersionText(self, text):
        self.version.setLabel(text)

    def _encode(self,text):
        try:
            return text.encode("utf-8")
        except Exception:
            return text
        
        
    def _nextPage(self):
        if(self._isNextPageAvailable()):
            page=self.movies["page"]+1
            if("next_page_token" in self.movies):
                self.pageToken=self.movies["next_page_token"]
            self._jumpTo(page)
    
    def _previousPage(self):
        if(self._isPrevPageAvailable()):
            page=self.movies["page"]-1
            if("prev_page_token" in self.movies):
                self.pageToken=self.movies["prev_page_token"]
            self._jumpTo(page)
            
    def _isNextPageAvailable(self):
        return self.movies["page"]<self.movies["pages_total"]
    
    def _isPrevPageAvailable(self):
        return self.movies["page"]>1
    
    def _isPageAvailable(self, page):
        return page>0 and page<=self.movies["pages_total"] 
    
    def _jumpTo(self, page):
        try:
            self._clearPanel()
            xbmc.executebuiltin( "ActivateWindow(busydialog)" )
            self._buildPanel(page=page)
        finally:
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
    
    def _clearPanel(self):
        for movie in self.movies["movies"]:
            if("control" in movie):
                try:
                    self.removeControl(movie["control"])
                except Exception:
                    pass
            if("button" in movie):
                try:
                    self.removeControl(movie["button"])
                except Exception:
                    pass
                
    def doJumpToDialog(self, text):
        dialog = xbmcgui.Dialog()
        ret=dialog.input("Перейти на страницу",text,type=xbmcgui.INPUT_NUMERIC)
        del dialog
        return (int)(ret)

    def doFav(self):
        list=None
        try:
            xbmc.executebuiltin( "ActivateWindow(busydialog)" )
            list=self.context.getApi().getFavorites(page=1,itemsPerPage=20)
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )
            if(list["count_total"]>0):
                pass
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok("Ошибка", "Список пуст")
                del dialog
                list=None       
        finally:
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
        return list
    
    def doGenre(self):
        list=None
        selected=None
        try:
            genres=self.context.getApi().getGenres()
            if(len(genres["genres"])==0):
                dialog = xbmcgui.Dialog()
                dialog.ok(const.TEXT_ERROR, "Список пуст")
                del dialog
            else:
                genres_array=[]
                for genre in genres["genres"]:
                    genres_array.append(self._encode(genre["genre"]))
                dialog = xbmcgui.Dialog()
                ret = dialog.select('Выбор жанра', genres_array)
                if(ret==-1):
                    return
                xbmc.executebuiltin( "ActivateWindow(busydialog)" )
                list=self.context.getApi().getMoviesByGenre(genreId=genres["genres"][ret]["id"], itemsPerPage=20)
                xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                if(list["count_total"]>0):
                    selected=genres["genres"][ret]
                else:
                    dialog = xbmcgui.Dialog()
                    dialog.ok(const.TEXT_SEARCH, const.TEXT_NOT_FOUND)
                    del dialog
        finally:
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
        return (list,selected)
        
    def doSearch(self):
        self.log("Searching")
        list=None
        old_search_text=None
        search_target=['Seasondream', 'YouTube','ВКонтакте']
        target_values=["default","youtube","vk"]
        dialog = xbmcgui.Dialog()
        ret = dialog.select('Искать в', search_target)
        if(ret==-1):
            return
        target=target_values[ret]
        while True:
            list=None
            keyboard = xbmc.Keyboard()
            if(self.search_object is not None):
                keyboard.setDefault(self.search_object["title"])
            if(old_search_text is not None):
                keyboard.setDefault(old_search_text)
            keyboard.setHeading(const.TEXT_SEARCH+" ["+search_target[ret]+"]")
            keyboard.doModal()
            
            if (keyboard.isConfirmed()):
                
                title=keyboard.getText()
                old_search_text=title
                actors=""
                year=""
                search_object={"title":title,"year":year,"actors":actors,"target":target}
                try:
                    xbmc.executebuiltin( "ActivateWindow(busydialog)" )
                    list=self.context.getApi().searchMovies(title=search_object["title"],year=search_object["year"],actors=search_object["actors"],target=search_object["target"],itemsPerPage=20)
                finally:
                    xbmc.executebuiltin( "Dialog.Close(busydialog)" )                

                if(list["count_total"]>0):
                    break
                else:
                    dialog = xbmcgui.Dialog()
                    dialog.ok(const.TEXT_SEARCH, const.TEXT_NOT_FOUND)
                    del dialog
            else:
                self.log("Search cancelled")
                break
            del keyboard
        if(list is not None):
            self.search_object=search_object
            
        return list        
        
    def getVersion(self):
        return "%.1f.%d%s"%(const.VERSION_MAJOR,const.VERSION_MINOR,const.VERSION_SUFFIX)                
                