# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..constants import const
from .BasePanel import BasePanel
from operator import itemgetter
import urllib2
import urllib
import xbmc, xbmcgui  # @UnresolvedImport
import re
from threading import Thread
from ..utils.Logger import Logger
import time
import os
import datetime
import traceback

from urlparse import urlparse

class EPGLoader(Thread, Logger):

    def __init__(self,context):
        Thread.__init__(self, name="EPGLoader")
        Logger.__init__(self)
        self.setDaemon(True)
        self.context=context
        
        self.path=self.context.getEPGPath()
        self.opener = urllib2.build_opener()

        if(os.path.isfile(os.path.join(self.path,"ready"))):
            os.remove(os.path.join(self.path, "ready"))
    
    def _ready(self, text=""):
        if(os.path.isfile(os.path.join(self.path,"ready"))):
            os.remove(os.path.join(self.path, "ready"))
        with open(os.path.join(self.path, "ready_temp"),"w") as f: 
            f.write(text)
        os.rename(os.path.join(self.path, "ready_temp"),os.path.join(self.path, "ready"))
    
        
    def run(self):
        if(self.context.getSetting("use_epg")=="true"):        
            try:
                _filename=os.path.join(self.path,"epg_"+datetime.date.today().strftime('%Y%m%d')+"_1.dat")
                ts_local=datetime.date.today()
                if(os.path.isfile(_filename)):
                    self._ready(text="1")
                    self.log("EPG for current day is available, skipping full waiting")
    
                ts_minus_14_days=(long)((ts_local+datetime.timedelta(days=-15)).strftime('%Y%m%d'))
                for name in os.listdir(self.path):
                    if os.path.isfile(os.path.join(self.path, name)) and name[-4:]==".dat":
                        try:
                            ts_file=(long)(name[4:12])
                        except Exception,e:
                            ts_file=None
                        if(ts_file and ts_minus_14_days>ts_file):
                            self.log("Removing "+name)
                            os.remove(os.path.join(self.path, name))
    
                url="http://seasondream.org/dl/epg/timestamp?t="+str(time.time())
                last_epg_update_server=self.opener.open(url).read()
                self.log("Last EPG update "+str(self.context.getSetting("last_epg_update")))
                if(self.context.getSetting("last_epg_update")==last_epg_update_server and os.path.isfile(os.path.join(self.path, "epg.zip"))):
                    self.log("EPG is up to date. Nothing to do.")
                    self._ready(text="1")
                else:
                    self.log("Start downloading EPG...")
                    try:
                        os.remove(self.path+"epg.zip")
                    except Exception, e:
                        self.log("Unable to remove epg.zip "+str(e))
                    self.context.setSetting("last_epg_update",ts_local.strftime('%Y%m%d'))                        
    
                    url="http://seasondream.org/dl/epg/epg.zip?t="+str(time.time())
                    urllib.urlretrieve(url, os.path.join(self.path,"epg.zip"))
                    self.log("Extracting files...")
                    try:
                        import zipfile                    
                        with zipfile.ZipFile(os.path.join(self.path,"epg.zip"), "r") as z:
                            z.extractall(self.path)        
                        self._ready(text="1")
                        self.log("Done")
                    except Exception,e:
                        self.log("Unable to extract zip file "+str(e))
                        self._ready(text="-1")
            except Exception,e:
                self.log("EPG Thread error "+str(e))
                self._ready(text=str(e))
        else:
            self._ready(text="-1")
            
        self.log("EPG Thread finished")

class PiconLoader(Thread, Logger):

    def __init__(self,context):
        Thread.__init__(self, name="PiconLoader")
        Logger.__init__(self)
        self.setDaemon(True)
        self.context=context
        self.opener = urllib2.build_opener()
        self.path=self.context.getPiconsPath()
        
    def run(self):
        self.log(str(self.context.getSetting("use_picons")))
        if(self.context.getSetting("use_picons")=="true"):
            try:
                url="http://seasondream.org/dl/picons/timestamp?t="+str(time.time())
                last_picons_update_server=self.opener.open(url).read()
                self.log("Last Picons update "+str(self.context.getSetting("last_picons_update"))+"|"+str(last_picons_update_server))
                if(self.context.getSetting("last_picons_update")==last_picons_update_server and os.path.isfile(os.path.join(self.path, "picons.zip"))):
                    self.log("Picons are up to date. Nothing to do.")
                else:
                    self.log("Start downloading Picons...")
                    try:
                        os.remove(os.path.join(self.path, "picons.zip"))
                    except Exception, e:
                        self.log("Unable to remove picons.zip "+str(e))
                    url="http://seasondream.org/dl/picons/picons.zip?t="+str(time.time())
                    urllib.urlretrieve(url, os.path.join(self.path, "picons.zip"))
                    self.log("Extracting files...")
                    try:
                        import zipfile                    
                        with zipfile.ZipFile(os.path.join(self.path, "picons.zip"), "r") as z:
                            z.extractall(self.path)
                            self.log("Done")
                    except Exception,e:
                        self.log("Unable to extract zip file "+str(e))
                    self.context.setSetting("last_picons_update",last_picons_update_server)
    
            except Exception,e:
                self.log("PiconLoader Thread error "+str(e))
        self.log("PiconLoader Thread finished")   

class IpTvPanel(BasePanel):
    FOCUS_CATEGORY_LIST=1
    FOCUS_CHANNEL_LIST=2
    FOCUS_PROGRAM_LIST=3
    FOCUS_BUTTON_PANEL=4
        
    def __init__(self, context):
        BasePanel.__init__(self,context=context,background='back3.png')
        self.channels=None
        self.channels_playlist=None
        self.current_external_cat_selected=None
        self.cat_selection_mode=const.CAT_SELECTION_MODE_INTERNAL #internal by default
        self.current_focus=0
        self.current_epg_date=datetime.date.today()
        self.catgory_epg_date=datetime.date.today()
        self.category_epg_cache={}
                    
        self.categoryList=xbmcgui.ControlList(0,62,250,620,selectedColor="0xFFFFA500",_itemHeight=27,_imageWidth=0,_imageHeight=0,_itemTextXOffset=4,_itemTextYOffset=-2)
        self.addControl(self.categoryList)

        self.channelList=xbmcgui.ControlList(250,62,500,620,selectedColor="0xFFFFA500",_itemHeight=45,_imageWidth=45,_imageHeight=45,_itemTextXOffset=4,_itemTextYOffset=-2, _space=5)
        self.addControl(self.channelList)

        self.programList=xbmcgui.ControlList(750,62,520,450,selectedColor="0xFFFFA500",_itemHeight=27,_imageWidth=0,_imageHeight=0,_itemTextXOffset=9,_itemTextYOffset=-2)
        self.addControl(self.programList)
        
        self.programDesc=xbmcgui.ControlTextBox(770,500,515,175)
        self.addControl(self.programDesc)
        self.genreButton.setEnabled(False)
        self.favButton.setEnabled(False)
        self.epg_status_timer=None
        self.epg_loaded=False
        
        for button in self.buttons:
            button.controlDown(self.categoryList)
            button.controlUp(self.categoryList)

    def _onEpgStatusTimer(self):
        self.log("Checking EPG load status...")
        if(os.path.isfile(os.path.join(self.context.getEPGPath(),"ready"))):
            with open(os.path.join(self.context.getEPGPath(),"ready")) as f:
                epg_status_text=f.read()
            self.epg_status_timer.stop()
            self.log("EPG has been loaded. Status "+str(epg_status_text))
            if(epg_status_text=="1"):
                xbmc.executebuiltin("Notification('EPG','EPG загружен',3000)")   
                self.epg_loaded=True
                for k in self.category_epg_cache.keys():# invalidate cache
                    if self.category_epg_cache[k] is None:
                        del self.category_epg_cache[k]
                self._reloadEpgForChannels()
            else:
                xbmc.executebuiltin("Notification('EPG','Ошибка загрузки EPG '"+str(epg_status_text)+",3000)")   
                self.log("Error while loading EPG")
        else:
            self.log("EPG has not loaded yet")
    
    def _getProgrammInfoForChannel(self, cat, ch, max_chars=None):
        if cat["number"]=="-2000":#favorites
            cat_epg=self.getEPGForCategory(ch["categories"][0],cat)
        else:
            cat_epg=self.getEPGForCategory(cat["number"],cat)
        p=None
        program_info=""
        if(cat_epg is not None):
            p,_=self._getProgrammForChannel(ch, catEpg=cat_epg)
        if(p is not None):
            program_info=datetime.datetime.fromtimestamp((long)(p["ut_start"])).strftime("%H:%M")
            program_info+="-"+datetime.datetime.fromtimestamp((long)(p["ut_stop"])).strftime("%H:%M")
            program_info+=" "+p["title"]
            if(max_chars is not None and len(program_info)>max_chars):
                return program_info[:max_chars]+"..."
            else:
                return program_info
        return None
    
    def _reloadEpgForChannels(self):
        self.log("Reloading EPG...")
        try:
            cat=self._getCats()[self.categoryList.getSelectedPosition()]
            for i,ch in enumerate(cat["channels"]):
                program_info=self._getProgrammInfoForChannel(cat,ch,50)
                if(program_info is not None):
                    item=self.channelList.getListItem(i)
                    label=item.getLabel()
                    data=label.split('[CR]')
                    if(len(data)==2):
                        item.setLabel(data[0]+"[CR][COLOR gray]"+program_info+"[/COLOR]")
                    elif(len(data)==1):
                        item.setLabel(data[0])
            for i in range(0, self.programList.size()):
                item=self.programList.getListItem(i)
                ut_start=item.getProperty("ut_start")
                if(ut_start is not None):
                    if(time.time()>(long)(ut_start)+15*60):
                        picon= os.path.join(self.context.createResourcePath('media','record.png'))
                        item.setArt({'thumb':picon })
            self.log("EPG has been reloaded")
        except Exception,e:
            self.log("Unable to reload EPG "+str(e))
            traceback.print_exc()            
            
                
    def _close(self):
        if(self.epg_status_timer is not None):
            self.epg_status_timer.stop()
        self.close()
    
    def init(self):
        try:
            PiconLoader(context=self.context).start()
            EPGLoader(context=self.context).start()
            
            self._buildInternalIPTvList()
            self._buildExternalIPTvList()
            
            #self.prevButton.setEnabled(False)
            #self.nextButton.setEnabled(False)
            self.tvButton.setEnabled(False)
            
            self.searchButton.setEnabled(False)

            self.playlistButton.setEnabled(True)
                
            if((self.context.getApi().internal_iptv==False and self.context.getApi().external_iptv==True )):
                if(len(self.channels_playlist["data"]["categories"])>1):
                    self.cat_selection_mode=const.CAT_SELECTION_MODE_EXTERNAL
                else:
                    try:
                        self.cat_selection_mode=const.CAT_SELECTION_MODE_EXTERNAL_SELECTED
                        self.channels_playlist["data"]["categories"][0]["external_cats"]=self._buildCatsFromPlaylist(self.channels_playlist["data"]["categories"][0]["pl"])
                        self.current_external_cat_selected=self.channels_playlist["data"]["categories"][0]["external_cats"]
                    except Exception:
                        self.cat_selection_mode=const.CAT_SELECTION_MODE_EXTERNAL
                        
            if(self.channels is None and self.channels_playlist is None):
                dialog = xbmcgui.Dialog()
                dialog.ok("Erorr", "IPTV portal is not available")
                del dialog
                return False
                        
            self.log("Cat selection mode set to "+str(self.cat_selection_mode))
            self._buildCategoryList()

            self.epg_status_timer=self.context.getTimer()
            self.epg_status_timer.callback.append(self._onEpgStatusTimer)
            self.epg_status_timer.start(1,False)
            
        except Exception,e:
            traceback.print_exc()
            self.log("Unable to initialize iptv "+str(e))
            return False
        return True
        
    def _getCats(self):
        if(self.cat_selection_mode==const.CAT_SELECTION_MODE_INTERNAL):
            selected_from=self.channels["data"]["categories"]
        elif(self.cat_selection_mode==const.CAT_SELECTION_MODE_EXTERNAL): 
            selected_from=self.channels_playlist["data"]["categories"]
        elif(self.cat_selection_mode==const.CAT_SELECTION_MODE_EXTERNAL_SELECTED): 
            selected_from=self.current_external_cat_selected
        return selected_from
    
    def _buildCategoryList(self):
        self.log("Building category list...")
        
        selected_from=self._getCats()

        self.categoryList.reset()
        self.channelList.reset()
        self.programList.reset()
        self.programDesc.setText("")

        for cat in selected_from:
            if("external" in cat and cat["external"]==True):
                pass
            else:
                if(cat["number"]=="103"):
                    continue
                elif(cat["number"]=="100"):
                    cat["title"]=const.TEXT_COMMON
                elif(cat["number"]=="105"):
                    cat["title"]=const.TEXT_EDUCATION
                elif(cat["number"]=="102"):
                    cat["title"]=const.TEXT_FILM
                elif(cat["number"]=="104"):
                    cat["title"]=const.TEXT_SPORT
                elif(cat["number"]=="102"):
                    cat["title"]=const.TEXT_CHILDREN
                elif(cat["number"]=="107"):
                    cat["title"]=const.TEXT_NEWS
                elif(cat["number"]=="108"):
                    cat["title"]=const.TEXT_MUSIC
            li=xbmcgui.ListItem(label=cat["title"])
            li.setProperty("number",cat["number"])
            self.categoryList.addItem(li)
            
        for i in range(0,self.categoryList.size()):
            self.categoryList.getListItem(i).select(False)

        self.categoryList.selectItem(0)
        self.categoryList.getListItem(0).select(True)
        self._onCategoryChanged(selected_from[0])
        self.setFocus(self.categoryList)
        self.current_focus=self.FOCUS_CATEGORY_LIST
        if(self.cat_selection_mode==const.CAT_SELECTION_MODE_EXTERNAL): 
            self.setTitleText("Мои плейлисты")
        else:
            self.setTitleText("")
            
        self.log("Done")
        
    def _buildChannelList(self, cat):
        self.log("Building channel list...")
        self.channelList.reset()
        self.programList.reset()
        self.programDesc.setText("")
        
        for ch in cat["channels"]:
            program_info=self._getProgrammInfoForChannel(cat,ch,50)
            if(program_info is not None):
                li=xbmcgui.ListItem(label=self._encode(ch["title"])+"[CR][COLOR gray]"+program_info+"[/COLOR]")
            else:
                li=xbmcgui.ListItem(label=self._encode(ch["title"]))
            
            li.setProperty("number", str(ch["number"]))            
            li.setProperty("title", self._encode(ch["title"]))
            picon= os.path.join(self.context.getPiconsPath(),"%d_%d_1_%s-or8.png"%(45,45,str(ch["number"])))
            li.setArt({'thumb':picon })
            self.channelList.addItem(li)
        if(len(cat["channels"])>0):
            self.channelList.selectItem(0)
            #self.channelList.getListItem(0).select(True)
            self._onChannelChanged(cat["channels"][0])
        
    def _onChannelChanged(self, channel):
        self._buildProgramList(channel)

    def _onProgramItemChanged(self, item):
        if(item.getProperty("description") is not None):
            self.programDesc.setText(item.getProperty("description"))

    def _onCategoryChanged(self, category):
        self._buildChannelList(cat=category)
    
    def _hasRecord(self, p, ch):
        try:
            if("source_url" in ch):
                return False # no records for external channels
            elif ("has_record" in p and "ut_start" in p and p["has_record"]==1):
                return time.time()>(long)(p["ut_start"])+15*60
        except Exception:
            pass
        return False
        
    def _buildProgramList(self, channel):
        self.log("Building program list...")
        self.programList.reset()
        self.programDesc.setText("")
        epg=self.getEPGForChannel(channel, self.current_epg_date)
        if(epg is not None):
            _curr_time=time.time()
            _index_to=0
            for i,p in enumerate(epg["programmes"]):
                text=datetime.datetime.fromtimestamp((long)(p["ut_start"])).strftime("%H:%M")+" "+self._encode(p["title"])
                li=xbmcgui.ListItem(label=text)
                if(self._hasRecord(p, channel)):
                    picon= os.path.join(self.context.createResourcePath('media','record.png'))
                    li.setArt({'thumb':picon })
                    li.setProperty("has_record","True")
                if("source_url" not in channel and "has_record" in p and p["has_record"]==1):
                    li.setProperty("ut_start",str(p["ut_start"]))
                    
                if("desc" in p):
                    desc=datetime.datetime.fromtimestamp((long)(p["ut_start"])).strftime("%H:%M")
                    desc+="-"+datetime.datetime.fromtimestamp((long)(p["ut_stop"])).strftime("%H:%M")
                    desc+=" "+self._encode(p["title"])+"\r\n"+"[COLOR gray]"+self._encode(p["desc"]+"[/COLOR]")
                    li.setProperty("description",desc)
                else:
                    li.setProperty("description","")
                self.programList.addItem(li)
                if(_curr_time>(long)(p["ut_start"])):
                    _index_to+=1                
        if(self.programList.size()>0):
            self.programList.selectItem(max(_index_to-1,0))
            for i in range(0,self.programList.size()):
                self.programList.getListItem(i).select(False)
            self._onProgramItemChanged(self.programList.getListItem(max(_index_to-1,0)))
        self.log("Done")
        
    def _isNextPageAvailable(self):
        if(self.epg_loaded==False or self.channelList.size()<1):
            return False
        current_epg_date_plus_1=self.current_epg_date + datetime.timedelta(days=1)
        ch = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
        if(self._checkEPGExists(ch,current_epg_date_plus_1)):
            return True
        else:
            return False
    
    def _isPrevPageAvailable(self):
        if(self.epg_loaded==False or self.channelList.size()<1):
            return False
        current_epg_date_minus_1=self.current_epg_date + datetime.timedelta(days=-1)
        ch = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
        if(self._checkEPGExists(ch,current_epg_date_minus_1)):
            return True
        else:
            return False

    def onAction(self, action):
        self.log("onAction %s"%str(action.getId()))
        if action.getId() in [const.ACTION_PREVIOUS_MENU,const.ACTION_NAV_BACK]:
            self.log("Closed")
            self._close()
            
        elif action.getId() in [const.ACTION_MOVE_DOWN,const.ACTION_MOVE_UP,const.ACTION_CHANNEL_DOWN,const.ACTION_CHANNEL_UP]:
            self.log("Current focus set to %d"%self.current_focus)
            if(self.current_focus==self.FOCUS_CATEGORY_LIST):
                control=self.categoryList
            elif(self.current_focus==self.FOCUS_CHANNEL_LIST):
                control=self.channelList
            elif(self.current_focus==self.FOCUS_PROGRAM_LIST):
                control=self.programList
            else:
                _focused=None
                try:
                    _focused=self.getFocus()
                except Exception:
                    pass
                if(_focused==self.categoryList):
                    control=self.categoryList
                    self.current_focus=self.FOCUS_CATEGORY_LIST                    
                elif(_focused==self.channelList):
                    control=self.channelList
                    self.current_focus=self.FOCUS_CHANNEL_LIST                    
                else:
                    return
            
            self.setFocus(control)
            selected_item=control.getSelectedPosition()
            self.log("Selected item %d"%selected_item)
            
            for i in range(0,control.size()):
                control.getListItem(i).select(False)
            control.selectItem(selected_item)
            control.getListItem(selected_item).select(True)

            if(self.current_focus==self.FOCUS_CATEGORY_LIST):
                if(self.cat_selection_mode in [const.CAT_SELECTION_MODE_INTERNAL,const.CAT_SELECTION_MODE_EXTERNAL_SELECTED]):
                    self._onCategoryChanged(self._getCats()[selected_item])
            elif(self.current_focus==self.FOCUS_CHANNEL_LIST):
                channel = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
                self._onChannelChanged(channel)
            elif(self.current_focus==self.FOCUS_PROGRAM_LIST):
                channel = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
                
                self._onProgramItemChanged(self.programList.getSelectedItem())
                
        elif action.getId() in [const.ACTION_MOVE_LEFT,const.ACTION_MOVE_RIGHT]:
            if(self.current_focus==self.FOCUS_CATEGORY_LIST and action.getId()==const.ACTION_MOVE_LEFT):
                self.current_focus=self.FOCUS_BUTTON_PANEL
                self.setFocus(self.playlistButton)
                self.categoryList.getListItem(self.categoryList.getSelectedPosition()).select(False)
                self.log("Focus switched to button panel")

            elif(self.current_focus==self.FOCUS_CATEGORY_LIST and action.getId()==const.ACTION_MOVE_RIGHT and self.channelList.size()>0):
                if(self.cat_selection_mode!=const.CAT_SELECTION_MODE_EXTERNAL):
                    self.current_focus=self.FOCUS_CHANNEL_LIST
                    self.setFocus(self.channelList)
                    self.categoryList.getListItem(self.categoryList.getSelectedPosition()).select(False)
                    self.channelList.getListItem(self.channelList.getSelectedPosition()).select(True)
                
            elif(self.current_focus==self.FOCUS_CHANNEL_LIST and action.getId()==const.ACTION_MOVE_RIGHT and self.programList.size()>0):
                self.current_focus=self.FOCUS_PROGRAM_LIST
                self.setFocus(self.programList)
                self.programList.getListItem(self.programList.getSelectedPosition()).select(True)
                self.channelList.getListItem(self.channelList.getSelectedPosition()).select(False)

            elif(self.current_focus==self.FOCUS_CHANNEL_LIST and action.getId()==const.ACTION_MOVE_LEFT and self.categoryList.size()>0):
                self.current_focus=self.FOCUS_CATEGORY_LIST
                self.setFocus(self.categoryList)
                self.categoryList.getListItem(self.categoryList.getSelectedPosition()).select(True)
                self.channelList.getListItem(self.channelList.getSelectedPosition()).select(False)

            elif(self.current_focus==self.FOCUS_PROGRAM_LIST and action.getId()==const.ACTION_MOVE_RIGHT and self.categoryList.size()>0):
                self.current_focus=self.FOCUS_CATEGORY_LIST
                self.setFocus(self.categoryList)
                self.categoryList.getListItem(self.categoryList.getSelectedPosition()).select(True)
                self.programList.getListItem(self.programList.getSelectedPosition()).select(False)
                
            elif(self.current_focus==self.FOCUS_PROGRAM_LIST and action.getId()==const.ACTION_MOVE_LEFT and self.channelList.size()>0):
                self.current_focus=self.FOCUS_CHANNEL_LIST
                self.setFocus(self.channelList)
                self.channelList.getListItem(self.channelList.getSelectedPosition()).select(True)
                self.programList.getListItem(self.programList.getSelectedPosition()).select(False)
            
    def _playlist(self):
        if(self.cat_selection_mode==const.CAT_SELECTION_MODE_INTERNAL and self.context.getApi().external_iptv==True):
            self.cat_selection_mode=const.CAT_SELECTION_MODE_EXTERNAL
        elif(self.cat_selection_mode == const.CAT_SELECTION_MODE_EXTERNAL_SELECTED): 
            self.cat_selection_mode=const.CAT_SELECTION_MODE_EXTERNAL
        elif(self.cat_selection_mode == const.CAT_SELECTION_MODE_EXTERNAL and self.context.getApi().internal_iptv==True): 
            self.cat_selection_mode=const.CAT_SELECTION_MODE_INTERNAL
        else:
            return
        self._buildCategoryList()
    
    def _nextPage(self):
        if(self.epg_loaded==False or self.cat_selection_mode!=const.CAT_SELECTION_MODE_INTERNAL):
            return
        if(self._isNextPageAvailable()):
            current_epg_date_plus_1=self.current_epg_date + datetime.timedelta(days=1)
            self.current_epg_date=current_epg_date_plus_1
            ch = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
            self._buildProgramList(ch)
    
    def _previousPage(self):
        if(self.epg_loaded==False or self.cat_selection_mode!=const.CAT_SELECTION_MODE_INTERNAL):
            return
        if(self._isPrevPageAvailable()):
            current_epg_date_minus_1=self.current_epg_date + datetime.timedelta(days=-1)
            self.current_epg_date=current_epg_date_minus_1
            ch = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
            self._buildProgramList(ch)

    def onControl(self,control):
        self.log("onControl "+str(control))
        if(self.prevButton==control):
            self._previousPage()
        elif(self.nextButton==control):
            self._nextPage()        
        elif(self.playlistButton==control):
            self._playlist()
        elif(self.channelList==control):
            # channel list
            selected_item=self.channelList.getSelectedPosition()
            self.log("Selected pos "+str(selected_item))
            channel = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][selected_item]
            self._playChannel(channel)
        elif(self.programList==control):
            # channel list
            selected_item=self.programList.getSelectedPosition()
            self.log("Selected pos "+str(selected_item))
            channel = self._getCats()[self.categoryList.getSelectedPosition()]["channels"][self.channelList.getSelectedPosition()]
            program_item=self.programList.getSelectedItem()
            if(program_item.getProperty("has_record") and program_item.getProperty("ut_start")):
                self._playChannel(channel,(long)(program_item.getProperty("ut_start")))
            else:
                self.log("No Record available")
                
        elif(self.categoryList==control):
            #category list
            if(self.cat_selection_mode==const.CAT_SELECTION_MODE_EXTERNAL):
                selected_item=self.categoryList.getSelectedPosition()
                self.current_playlist=self.channels_playlist["data"]["categories"][selected_item]
                if(self.current_playlist["external_cats"] is None):
                    try:
                        xbmc.executebuiltin( "ActivateWindow(busydialog)" )
                        self.current_playlist["external_cats"]=self._buildCatsFromPlaylist(self.current_playlist["pl"])
                        xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                    except Exception,e:
                        xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                        dialog = xbmcgui.Dialog()
                        dialog.ok("Erorr", str(e))
                        del dialog
                        return
                if(len(self.current_playlist["external_cats"])==0):
                        dialog = xbmcgui.Dialog()
                        dialog.ok("Erorr", "Playlist is empty")
                        del dialog
                        return
                self.cat_selection_mode=const.CAT_SELECTION_MODE_EXTERNAL_SELECTED
                self.current_external_cat_selected=self.current_playlist["external_cats"]                
                self._buildCategoryList()            

    def hasExternalEPG(self, channel):
        return "number_external" in channel and channel["number_external"] is not None

    def _checkEPGExists(self,channel,date=None):
        if(self.context.getSetting("use_epg")!="true"):
            return None
        
        if(date is None):
            date=datetime.date.today()
        if(self.hasExternalEPG(channel)):
            _filename=os.path.join(self.context.getEPGPath(),"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["channel_provider"])+"_"+str(channel["number_external"])+".dat")
        else:
            if(channel["number"]==-1):
                return None
            _filename=os.path.join(self.context.getEPGPath(),"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["number"])+".dat")
        if(os.path.isfile(_filename)):
            return _filename
        else:
            return None
    
    def getEPGForChannel(self, channel, date=None, external=False):
        if(date is None):
            date=datetime.date.today()
        _filename=self._checkEPGExists(channel, date)
        if(_filename is not None):
            self.log("Loading "+_filename)
            with open(_filename,"r") as f:
                dat=f.read()
            try:
                ret=eval(dat)
                self.log("EPG "+_filename+" loaded")
                return ret
            except Exception,e:
                self.log("Error while loading EPG file. Corrupted data? "+str(e))
                return None
        else:
            if(self.hasExternalEPG(channel)):
                self.log("External EPG "+"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["number_external"])+".dat"+" is not available")
            else:
                self.log("EPG "+"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["number"])+".dat"+" is not available")
            return None

    def _getProgrammForChannel(self, channel_data, curr_time=None, catEpg=None):
        if(self.hasExternalEPG(channel_data)):
            channel=channel_data["number_external"]
            external=True
        else:
            channel=channel_data["number"]
            external=False
        if(curr_time is None):
            curr_time=time.time()
        date=datetime.datetime.fromtimestamp(curr_time)
        if(catEpg is None):
            _epg=self.getEPGForChannel(channel_data, date, external)
        else:
            _epg=None
            for ch in catEpg:
                if(str(ch["number"])==str(channel)):
                    _epg=ch
                    break
        
        if(_epg is not None):
            current_prog=None
            next_prog=None
            c=0
            for p in _epg["programmes"]:
                if(current_prog is not None):
                    next_prog=p
                    break        
                _ut_start=(long)(p["ut_start"])
                _ut_end=(long)(p["ut_stop"])
                if(curr_time>=_ut_start and curr_time<_ut_end):
                    _duration=_ut_end-_ut_start
                    p["duration"]=_duration
                    p["percent"]=(curr_time-_ut_start)*100/_duration
                    current_prog=p
                c+=1
            if(current_prog is not None and next_prog is None):
                _epg=self.getEPGForChannel(channel_data, date=datetime.datetime.fromtimestamp((int)(current_prog["ut_stop"])))
                if(_epg is not None and len(_epg["programmes"])>0):
                    for p in _epg["programmes"]:
                        if(((long)(p["ut_start"])) != ((long)(current_prog["ut_start"]))): 
                            next_prog=p
                            self.log("Program next (day+1) " +datetime.datetime.fromtimestamp(next_prog["ut_start"]).strftime("%d.%m.%Y %H:%M:%S")+" "+datetime.datetime.fromtimestamp(next_prog["ut_stop"]).strftime("%d.%m.%Y %H:%M:%S"))
                            break
            
            return (current_prog, next_prog)
            
        else:
            self.log("EPG not found for channel "+str(channel))
            return (None, None)

    def _loadExternalEPG(self, ch_array, date):
        if(self.context.getSetting("use_epg")!="true"):
            return None
        return self.context.getApi().ipTvGetExternalEPG(ch_array, date)

    def _checkEPGCategoeyExists(self,category, date=None):
        if(self.context.getSetting("use_epg")!="true"):
            return None

        if(date is None):
            date=datetime.date.today()
        _filename=os.path.join(self.context.getEPGPath(),"epg_"+date.strftime('%Y%m%d')+"_"+str(category)+"_cat.dat")
        if(os.path.isfile(_filename)):
            return _filename
        else:
            return None

    def loadEPGForCategory(self, category, date=None, cat=None):
        if(date is None):
            date=datetime.date.today()
        if((int)(cat["number"])>=2000000):
            #rest
            self.log("Building channel array for category others..")
            ch_array=[]
            for ch in cat["channels"]:
                if(self.hasExternalEPG(ch)):
                    ch_array.append(str(ch["number_external"]))
            if(len(ch_array)>0):
                dat=self._loadExternalEPG(ch_array, date)
                return dat
                
            else:
                return None
        else:
            _filename=self._checkEPGCategoeyExists(category, date)
            if(_filename is not None):
                self.log("Loading "+_filename)
                with open(_filename,"r") as f:
                    dat=f.read()
                try:
                    ret=eval(dat)
                    self.log("EPG Cateory "+_filename+" loaded")
                    return ret
                except Exception,e:
                    self.log("Error while loading EPG file. Corrupted data? "+str(e))
                    return None
                    
            else:
                self.log("EPG "+"epg_"+date.strftime('%Y%m%d')+"_"+str(category)+"_cat.dat"+" is not available")
                return None


    def getEPGForCategory(self, category, cat):
        if(datetime.date.today() != self.catgory_epg_date):
            self.catgory_epg_date=datetime.date.today()
            self.category_epg_cache={}
        if(category in self.category_epg_cache):
            return self.category_epg_cache[category]
        self.log("Loading cat EPG..")
        epg=self.loadEPGForCategory(category, cat=cat)
        self.category_epg_cache[category]=epg
        self.log("Cat EPG loaded")
        return epg

    
    def _checkExternalUrl(self, url):
        self.log("Checking "+str(url))
        if url.startswith("http://glaz.tv"):
            self.log("Glaz.tv url detected")
            try:
                url=self.context.getApi().getPlaylist(url,1003)
            except Exception,e:
                self.log("Unable to convert glaz.tv url "+str(e))
                return (None,None)
            return (url,const.MODE_M3U8_REGULAR)
        elif url.startswith("http://www.filmon.com/tv/api/channel/"):
            self.log("Filmon url detected")
            return (url,const.MODE_M3U8_FILMON)
        elif re.compile('/udp/[0-9]{1,3}[.][0-9]{1,3}[.][0-9]{1,3}[.][0-9]{1,3}[:][0-9]{1,4}\s*$', re.IGNORECASE).search(url):
            self.log("Udpxy url detected")
            return (url,const.MODE_PROXY_UDPXY)
        elif re.compile('/live/([^/]+?)/([^/]+?)/[0-9]{1,100}[.]ts\s*$', re.IGNORECASE).search(url):
            self.log("Xtream-Codes url detected")
            try:
                import mmap  # @UnusedImport
                self.log("mmap is installed")
                return (url,const.MODE_XTREAM_CODES)
            except Exception:
                self.log("mmap seems not to be installed. Can't use xtream proxy.")
                return (url, None)
        self.log("No handler found")
        return (url, None)
    
    def _playChannel(self, channel, start=None):
        number=channel["number"]
        title=channel["title"]
        self.log("Loading url for ch number %s timestamp %s"%(str(number),str(start)))
        if("source_url" in channel):
            self.log("Playing external")
            url, mode=self._checkExternalUrl(channel["source_url"])
            if(url is None):
                dialog = xbmcgui.Dialog()
                dialog.ok("Erorr", "Unable to play: no reasonable URL found.")
                del dialog
                return
            if(mode is not None):
                streamer=self.context.getStreamer(url=url, mode=mode)
            else:
                streamer=None
        else:
            self.log("Playing internal")
            if(start is not None):
                self.log("Playing record " +datetime.datetime.fromtimestamp(start).strftime("%d.%m.%Y %H:%M:%S"))
                
            url=self.context.getApi().ipTvGetUrl(number=number, start=start)["data"]["url"]
            streamer=self.context.getStreamer(url=url, mode=const.MODE_M3U8_INTERNAL)
        
        if(streamer is not None):
            port=streamer.init()
            if(port!=-1):
                self.log("Server started on port "+str(port))
                url="http://127.0.0.1:%d/stream.ts"%port                
                streamer.start()
            else:
                self.log("Unable to start streamer")  
                dialog = xbmcgui.Dialog()
                dialog.ok("Erorr", "Unable to start streamer")
                del dialog
                return
            
        pl=xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
        pl.clear()            
        listitem = xbmcgui.ListItem(title)
        listitem.setMimeType("application/octet-stream") #still sends HEAD request....
        listitem.setProperty('mimetype', 'application/octet-stream')
        picon= os.path.join(self.context.getPiconsPath(),"%d_%d_1_%s-or8.png"%(100,100,str(channel["number"])))
        listitem.setArt({'poster':picon,'banner':picon})
        
        pl.add(url, listitem)            
        xbmc.Player().play(pl)            

            
    def _buildInternalIPTvList(self):
        if(self.context.getApi().internal_iptv==False):
            return
        try:
            self.context.getApi().ipTvLogin()
        except Exception,e:        
            self.log("Unable to login %s"%str(e))
            return
        self.log("Loading cahannel data..")
        self.channels=self.context.getApi().ipTvGetChannels()
        self.log("Channel loaded")
        self.log(str(self.channels))
        ch_hd=[]
        cat_hd={"number":"-1000","title":const.TEXT_ALL_HD,"channels":None}
        self.channels["data"]["categories"].append(cat_hd)
        
        for cat in self.channels["data"]["categories"]:
            cat["external"]=False #we are internal
            if(cat["number"]=="103"):
                continue
            res=[]
            for ch in self.channels["data"]["channels"]:
                if("title" in ch):
                    ch["title"]=re.sub(r"[(].*[)]", r"", ch["title"])
                if cat["number"] in ch["categories"]:
                    res.append(ch)
            cat["channels"]=sorted(res, key=itemgetter("title"))

        for cat in self.channels["data"]["categories"]:
            for ch in cat["channels"]:
                if "is_hd" in ch and ch["is_hd"]=="1" and cat["number"]!="-2000":
                    ch_hd.append(ch)
        cat_hd["channels"]=ch_hd
        self.log("Internal playlist built")
        
    def _buildExternalIPTvList(self):
        if(self.context.getApi().external_iptv==False):
            return
        for pl in self.context.getApi().getIpTvList()["playlist"]:
            try:
                if("TYPE" not in pl):
                    pl["TYPE"]=1
                if(self.channels_playlist is None):
                    self.channels_playlist={"data":{"categories":[]}}
                cat={"external":False,"playlist":True, "number":"-100","title":pl["NAME"],"channels":[],"pl":pl,"external_cats":None}# empty channel category
                self.channels_playlist["data"]["categories"].append(cat)               
            except Exception,e:
                self.log("Unable to build playlist "+str(e))

    def _getUrlSuffix(self, url, default=""):
        try:
            parsed=urlparse(url)
            if(parsed.path.rindex(".")>-1):
                return parsed.path[parsed.path.rindex(".")-len(parsed.path):] 
        except Exception:
            pass
        return default
    
    def _buildCatsFromPlaylist(self, pl):
        self.log("Building cat from a external playlist...")
        opener = urllib2.build_opener()
        opener.addheaders = [('User-Agent', const.DEFAULT_USER_AGENT),("Cache-Control","no-cache")]
        external_cats=self._buildCatsFromPlaylistContent(content=opener.open(pl["URL"]).read(), suffix=self._getUrlSuffix(pl["URL"],".m3u8"), type=pl["TYPE"])
        self.log("External cat has been built #"+str(len(external_cats)))
        return external_cats
    
    def _buildCatsFromPlaylistContent(self, content, suffix=".m3u", type=1):
        if(type==2):
            _folders, _files=self.context.getApi().parseFilmonContent(cache_file=None)
        elif(suffix in [".m3u", ".m3u8"]):
            _folders, _files=self.context.getApi().parseM3U8Content(content,"","", files_only=True)
        elif(suffix in [".xml"]):
            _folders, _files=self.context.getApi().parseXMLContent(content,"","", files_only=True)
        else:
            self.log("Unknown suffix "+str(suffix)+", treat it like m3u")
            _folders, _files=self.context.getApi().parseM3U8Content(content,"","", files_only=True)

        _temp_cat={}
        cat_array=[]
        _id_counter=2000000
        for _file in _files:
            _use_cat_mapping=True
            if ("SEASONDREAM_IGNORE_CAT_MAPPING" in _file and _file["SEASONDREAM_IGNORE_CAT_MAPPING"]==True):
                _use_cat_mapping=False
            if(_use_cat_mapping==True):
                try:
                    extended_data=self.context.getApi().getPlaylist(_file["NAME"],1000)
                except Exception:
                    extended_data={'chid':-1,'catid':-2,'catname':const.TEXT_REST}
            else:
                extended_data={'chid':-1,'catid':-2,'catname':const.TEXT_REST}
            self.log(_file["NAME"] +" mapped to "+str(extended_data))
            if(extended_data['catid']<-1):
                extended_data["catid"]=_id_counter
            _cat_name=extended_data['catname']
            if(extended_data['catid']>=2000000):
                if("group" in _file and _file["group"] is not None):
                    _cat_name= _file["group"]
                
                
            if(_cat_name not in _temp_cat):
                cat={"external":True, "number":str(extended_data['catid']),"title":_cat_name,"channels":[]}
                _temp_cat[_cat_name]=cat
                cat_array.append(cat)
            
            _channel={"number":extended_data['chid'],"number_external":None,"channel_provider":None, "title":_file["NAME"],"source_url":_file["SOURCE_URL"],"extvlcopt":_file["EXTVLCOPT"],"seasondream_type":_file["SEASONDREAM_TYPE"],"seasondream_reload_intervall":_file["SEASONDREAM_RELOAD_INTERVALL"],"seasondream_ignore_m3u8_eol":_file["SEASONDREAM_IGNORE_M3U_EOL"]}
            
            if(extended_data['catid']>=2000000 and _use_cat_mapping==True):
                try:
                    mapped_to_epg=self.context.getApi().getPlaylist(_file["NAME"],1001)
                    _channel["number_external"]=mapped_to_epg["chid"]
                    _channel["channel_provider"]=mapped_to_epg["channel_provider"]
                    def downloadExternalEPG():
                        if(self.context.getSetting("use_epg")==False):
                            return None
                        try:
                            def _process(url, dst):
                                bs = 1024*256
                                opener=urllib2.build_opener()
                                opener.addheaders = [('User-Agent', const.DEFAULT_USER_AGENT)]
                                fp=opener.open(url)
                                with open(dst, "wb") as tfp:
                                    while 1:
                                        block = fp.read(bs)
                                        if block == "":
                                            break
                                        tfp.write(block)
                                fp.close()
                            date=datetime.date.today()
                            _filename=os.path.join(self.context.getEPGPath(),"epg_"+date.strftime('%Y%m%d')+"_"+str(_channel["channel_provider"])+"_"+str(_channel["number_external"])+".dat")
                            if(os.path.isfile(_filename)):
                                self.log("External EPG for "+str(_channel["number_external"])+" already exists")
                                return
                            _epg_url="http://api.seasondream.org/epg.php?channels="+str(_channel["number_external"])+"&date="+date.strftime('%Y%m%d')
                            t=Thread(target=_process,args=(_epg_url, _filename))
                            t.setDaemon(True)
                            t.start()                            
                        except Exception,e:
                            self.log("Unable to download EPG "+str(e))
                    downloadExternalEPG()
                except Exception:
                    pass
            
            cat=_temp_cat[_cat_name]
            cat["channels"].append(_channel)
            _id_counter+=1

        for cat in cat_array:
            cat["channels"]=sorted(cat["channels"], key=itemgetter("title"))         
        cat_array=sorted(cat_array, key=itemgetter("title"))    
        return cat_array
    
            