# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..utils.Text import text
from ..constants import const
from .MoviePanel import MoviePanel
from .BasePanel import ControlSliderStar
from .BasePanel import BasePanel
from .IpTvPanel import IpTvPanel

import xbmc, xbmcgui  # @UnresolvedImport

class LatestMoviesPanel(BasePanel):
    
    def __init__(self, context, page=1):
        BasePanel.__init__(self,context=context,background='back1.png')

        self.movies=None
        self.selected_movie=None
        self.search_object=None
        self.button_array={}
        self.control_array={}
        

        self.movieDescription=xbmcgui.ControlTextBox(17,443,604,210)
        self.addControl(self.movieDescription)

        self.movieInfo=xbmcgui.ControlTextBox(660,443,604,210)
        self.addControl(self.movieInfo)

        self.movieTitle=xbmcgui.ControlLabel(17,397,604,25,'',font='font14',alignment=0,textColor='0xFFFFA500')
        self.addControl(self.movieTitle)

        self.provider=xbmcgui.ControlLabel(925,398,200,25,'',font='font13',alignment=1,textColor='0xFFFFA500') #1-right, 2-center
        self.addControl(self.provider)
        
        self.rating=ControlSliderStar(1140,403,context,self)

        self._buildPanel(page=page)
    
    def onControl(self,control):
        self.log("onControl "+str(control))
        if(self.prevButton==control):
            self._previousPage()
        elif(self.nextButton==control):
            self._nextPage()
        elif(self.searchButton==control):
            self._search()
        elif(self.tvButton==control):
            self._tv()
        elif(self.genreButton==control):
            self._genre()
        elif(self.favButton==control):
            self._fav()
        else:
            movie=self._getMovieByControlId(control.getId())
            if(movie is not None):
                ar=[]
                ar.append(movie)
                mp=MoviePanel(context=self.context, movies={"movies":ar,"page":1,"pages_total":1},action=const.SD_ACTION_SINGLE)
                mp.doModal()
                del mp
            else:
                self.log("No movie for controlId %d found"%control.getId())            
    
    # wont fire by kodi...don't know why
    def onFocus(self, controlId):
        #self.log("onFocus "+str(controlId))
        movie=self._getMovieByControlId(controlId)
        if(self.selected_movie is None or self.selected_movie!=movie):
            self._onMovieItemChanged(movie)
            self.selected_movie=movie
    
    def onAction(self, action):
        self.log("onAction "+str(action.getId()))
        if action.getId() in [const.ACTION_PREVIOUS_MENU,const.ACTION_NAV_BACK]:
            self.log("Closed")
            self.context.hideSplashScreen()
            self.close()
        elif action.getId() in [const.ACTION_MOUSE_MOVE,const.ACTION_MOVE_DOWN,const.ACTION_MOVE_LEFT,const.ACTION_MOVE_RIGHT,const.ACTION_MOVE_UP]:
            self.onFocus(self.getFocusId())
        elif action.getId() in [const.REMOTE_1,const.REMOTE_2,const.REMOTE_3,const.REMOTE_4,const.REMOTE_5,const.REMOTE_6,const.REMOTE_7,const.REMOTE_8,const.REMOTE_9]:
            ret=self.doJumpToDialog(str(action.getId()-const.REMOTE_0))
            if(ret!=-1 and self._isPageAvailable(ret)):
                self._jumpTo(ret)
        elif action.getId() ==const.ACTION_PREV_ITEM:
            self.onControl(self.prevButton)
        elif action.getId() ==const.ACTION_NEXT_ITEM:
            self.onControl(self.nextButton)
            
            
    def _search(self):
        _list=self.doSearch()
        if(_list is not None):
            #we've found someting, display movie detail page
            mp=MoviePanel(context=self.context,movies=_list,action=const.SD_ACTION_SEARCH,search_object=self.search_object)
            mp.doModal()
            del mp
        else:
            pass
      
    def _onMovieItemChanged(self, movie):
        if(movie is not None):
            self.movieDescription.setText(self._encode(movie["DESCRIPTION"]))
            try:
                self.movieDescription.autoScroll(10000, 5000, 1000)
            except Exception:
                pass
            self.movieInfo.setText("[COLOR gray]"+text.getMovieInfoShort(movie)+"[/COLOR]")
            self.movieTitle.setLabel(self._encode(movie["TITLE"]))
            self.provider.setLabel(self.context.getApi().getPluginByKey(movie["PROVIDER_ID"]))
            rating=text.getRating(movie)
            if(rating):
                self.rating.show()
                self.rating.setValue(rating)
            else:
                self.rating.hide()
 
    def _loadMovies(self, page=1):
        if(self.movies is None or self.movies["page"]!=page):
            self.log("Loading movies for page %d"%page)
            self.movies=self.context.getApi().getNewMovies(page=page)
            self.log("Movielist has been loaded. Page %d"%self.movies["page"])
            
        self.nextButton.setEnabled(self._isNextPageAvailable())
#        self.nextButton.setVisible(self._isNextPageAvailable())

        self.prevButton.setEnabled(self._isPrevPageAvailable())
        #self.prevButton.setVisible(self._isPrevPageAvailable())
        return self.movies
    
    def _getMovieByControlId(self, controlId):
        for movie in self.movies["movies"]:
            if("button" in movie and movie["button"].getId()==controlId):
                return movie
        return None

    def _clearPanel(self):
        self.selected_movie=None
    
    def _createControl(self,p,x,y,w,h):
        if(p not in self.control_array):
            self.control_array[p]=xbmcgui.ControlImage(x,y,w,h,'')
            self.addControl(self.control_array[p])
        return self.control_array[p]

    def _createButton(self,p,x,y,w,h,label,sel,nonsel):
        if(p not in self.button_array):
            self.button_array[p]=xbmcgui.ControlButton(x,y,w,h,label,sel,nonsel)
            self.addControl(self.button_array[p])
        return self.button_array[p]
        
    def _buildPanel(self, page=1):
        column=0
        row=0
        IMG_WIDTH=110
        IMG_HEIGHT=155
        TOP=52
        LEFT=25
        BORDER=10
        ITEMS_PER_ROW=10

        for i,movie in enumerate(self._loadMovies(page=page)["movies"]):
            x=LEFT+(column*IMG_WIDTH)+BORDER*(column+1)
            y=TOP+(row*IMG_HEIGHT)+BORDER*(row+1)
            
            movie["control"]=self._createControl(i,x,y,IMG_WIDTH,IMG_HEIGHT)
            movie["button"]=self._createButton(i,x-BORDER/2-5,y-BORDER/2-5,IMG_WIDTH+BORDER+10,IMG_HEIGHT+BORDER+10,u'',self.context.createResourcePath('media','picture_border.png'),'')
            
            self.context.populateImage(movie)
            column+=1
            if(column>ITEMS_PER_ROW-1):
                row+=1
                column=0
        # set navigation
        movie_len=len(self.movies["movies"])
        column=0
        row=0
        
        for i in range(0,movie_len):
            right=(i+1)%movie_len
            left=(movie_len+i-1)%movie_len
            down=i%ITEMS_PER_ROW
            if(i<ITEMS_PER_ROW):
                down+=ITEMS_PER_ROW #row1
            else:
                pass #row2
            
            up=i%ITEMS_PER_ROW
            if(i<ITEMS_PER_ROW):
                up+=ITEMS_PER_ROW
            if(right<movie_len):
                self.movies["movies"][i]["button"].controlRight(self.movies["movies"][right]["button"])
            if(left<movie_len):
                self.movies["movies"][i]["button"].controlLeft(self.movies["movies"][left]["button"])
            if(up<movie_len):
                self.movies["movies"][i]["button"].controlUp(self.movies["movies"][up]["button"])
            if(down<movie_len):
                self.movies["movies"][i]["button"].controlDown(self.movies["movies"][down]["button"])
            #overwrite down for last row
            if(row==movie_len/ITEMS_PER_ROW-1): #last row
                self.movies["movies"][i]["button"].controlDown(self.searchButton)

            column+=1
            if(column>ITEMS_PER_ROW-1):
                row+=1
                column=0
        
        if(movie_len>0):
            for button in self.buttons:
                button.controlDown(self.movies["movies"][0]["button"])
                button.controlUp(self.movies["movies"][0]["button"])
            
            self.setFocus(self.movies["movies"][0]["button"])
            #fire focus event
            self.onFocus(self.movies["movies"][0]["button"].getId())
            
        self.setTitleText("%s (%d/%d)"%(const.TEXT_LATESTS,self.movies["page"],self.movies["pages_total"]))
        
    def _fav(self):
        _list=self.doFav()
        if(_list is not None):
            mp=MoviePanel(context=self.context,movies=_list,action=const.SD_ACTION_FAV,search_object=self.search_object)
            mp.doModal()
            del mp
        
    def _genre(self):
        _list,genre=self.doGenre()
        if(genre is not None):
            mp=MoviePanel(context=self.context,movies=_list,action=const.SD_ACTION_GENRE,search_object=self.search_object, genre=genre)
            mp.doModal()
            del mp
            
    
    def _tv(self):
        xbmc.executebuiltin( "ActivateWindow(busydialog)" )
        try:
            self.context.getApi().ipTvCheckCredentials()
            if(self.context.getApi().internal_iptv==False and self.context.getApi().external_iptv==False):
                dialog = xbmcgui.Dialog()
                dialog.ok(const.TEXT_SEARCH, const.TEXT_IPTV_NOT_AVAILABLE)
                del dialog
                return
            mp=IpTvPanel(context=self.context)
            if(mp.init()==True):
                xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                mp.doModal()
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
            del mp
        finally:
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                
        