# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..constants import const
from .BasePanel import BasePanel
from .IpTvPanel import IpTvPanel
import xbmc, xbmcgui  # @UnresolvedImport
from ..utils.Text import text

class MoviePanel(BasePanel):
    FOCUS_MOVIE_LIST=1
    FOCUS_PLAY_LIST=2
    FOCUS_BUTTON_PANEL=3
    
    def __init__(self, context, movies, action, search_object={"title":"","year":"","actors":"","target":"default"}, genre=None):
        BasePanel.__init__(self, context=context, background='back2.png')
        self.genre=genre
        self.movies=movies
        self.action=action
        self.search_object=search_object
        
        self.image=xbmcgui.ControlImage(556,63,177,265,'')
        self.addControl(self.image)

        self.movieInfo=xbmcgui.ControlTextBox(738,85,533,265)
        self.addControl(self.movieInfo)

        self.movieList=xbmcgui.ControlList(10,62,500,610,font='font13',selectedColor="0xFFFFA500",_itemHeight=27,_imageWidth=16,_imageHeight=16,_itemTextXOffset=4,_itemTextYOffset=-2)
        self.addControl(self.movieList)
 
        self.playList=xbmcgui.ControlList(517,544,703,133,selectedColor="0xFFFFA500",_itemHeight=22)
        self.addControl(self.playList)

        self.movieDescription=xbmcgui.ControlTextBox(556,350,703,163)
        self.addControl(self.movieDescription)

        self.playListText=xbmcgui.ControlLabel(550,564,703,113,'',font='font13',alignment=2) #1-right, 2-center
        
        self.addControl(self.playListText)

        self.provider=xbmcgui.ControlLabel(1070,57,200,25,'',font='font13',alignment=1,textColor='0xFFFFA500') #1-right, 2-center
        self.addControl(self.provider)

        self.movieTitle=xbmcgui.ControlLabel(738,57,400,25,'',font='font13',alignment=0,textColor='0xFFFFA500') #1-right, 2-center
        self.addControl(self.movieTitle)
        
        

        for button in self.buttons:
            button.controlDown(self.movieList)
            button.controlUp(self.movieList)

        self.current_focus=0
        
        self._buildPanel()
        
    def onControl(self,control):
        self.log("onControl "+str(control))
        if(self.prevButton==control):
            self._previousPage()
        elif(self.nextButton==control):
            self._nextPage()
        elif(self.searchButton==control):
            self._search()
        elif(self.tvButton==control):
            self._tv()
        elif(self.genreButton==control):
            self._genre()
        elif(self.favButton==control):
            self._fav()
        
        elif control == self.movieList:
            item = self.movieList.getSelectedItem()
            movie = self._getMovieForItem(item)

            self.log('Loading playlist %s'%item.getLabel())
            try:
                xbmc.executebuiltin( "ActivateWindow(busydialog)" )
                self._loadPlaylist(movie)
            finally:
                xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
            
        elif control == self.playList:
            movie = self._getMovieForItem(self.movieList.getSelectedItem())
            item = self.playList.getSelectedItem()

            self.log('Playing movie %s'%item.getLabel())
            item=self.pl[self.playList.getSelectedPosition()]

            if("future" in item):
                item["file"]=item["future"]()
            if("type" in item):
                if("M3U8"==item["type"]):
                    item["file"]=item["media_url"]
                else:
                    self.log("Unknown media type %s"%item["type"])
                    
            self.log("File %s"%item["file"])
            pl=xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
            pl.clear()            
            listitem = xbmcgui.ListItem(self._encode(item["comment"]))
            listitem.setInfo('video',{'title':self._encode(movie["TITLE"])})
            if(self.action==const.SD_ACTION_SEARCH and self.search_object["target"] in ["youtube"]):
                picture=self.context.createResourcePath('media','youtube.png')
            elif(self.action==const.SD_ACTION_SEARCH and self.search_object["target"] in ["vk"]):
                picture=self.context.createResourcePath('media','vk.png')
            else:
                picture=self.context.getImageForMovie(movie)
            listitem.setInfo('picture',{'picturepath':picture})
            listitem.setArt({'poster':picture,'banner':picture})
            
            pl.add(item["file"], listitem)            
            xbmc.Player().play(pl)            
    
    def onAction(self, action):
        self.log("onAction %s"%str(action.getId()))
        if action.getId() in [const.ACTION_PREVIOUS_MENU,const.ACTION_NAV_BACK]:
            self.log("Closed")
            self.close()
            
        elif action.getId() in [const.ACTION_MOVE_DOWN,const.ACTION_MOVE_UP,const.ACTION_CHANNEL_DOWN,const.ACTION_CHANNEL_UP]:
            self.log("Current focus set to %d"%self.current_focus)
            if(self.current_focus==self.FOCUS_MOVIE_LIST):
                control=self.movieList
            elif(self.current_focus==self.FOCUS_PLAY_LIST):
                control=self.playList
            else:
                _focused=None
                try:
                    _focused=self.getFocus()
                except Exception:
                    pass
                if(_focused==self.movieList):
                    control=self.movieList
                    self.current_focus=self.FOCUS_MOVIE_LIST                    
                elif(_focused==self.movieList):
                    control=self.playList
                    self.current_focus=self.FOCUS_PLAY_LIST                    
                else:
                    return
            
            self.setFocus(control)
            selected_item=control.getSelectedPosition()
            self.log("Selected item %d"%selected_item)
            
            for i in range(0,control.size()):
                control.getListItem(i).select(False)
            control.selectItem(selected_item)
            control.getListItem(selected_item).select(True)

            if(self.current_focus==self.FOCUS_MOVIE_LIST):
                item = self.movieList.getSelectedItem()
                movie = self._getMovieForItem(item)
                self._onMovieItemChanged(movie)
            elif(self.current_focus==self.FOCUS_PLAY_LIST):
                self._onPlaylistItemChanged()
            
        elif action.getId() in [const.ACTION_MOVE_LEFT,const.ACTION_MOVE_RIGHT]:
            if(self.current_focus==self.FOCUS_PLAY_LIST):
                self.current_focus=self.FOCUS_MOVIE_LIST
                self.playListText.setVisible(True)
                self.playListText.setLabel(const.TEXT_PRESS_OK_FOR_LOADING_PLAYLIST)
                self.playList.reset()
                self.setFocus(self.movieList)
                self.movieList.getListItem(self.movieList.getSelectedPosition()).select(True)
                
            elif(self.current_focus==self.FOCUS_MOVIE_LIST and action.getId()==const.ACTION_MOVE_LEFT):
                self.current_focus=self.FOCUS_BUTTON_PANEL
                self.setFocus(self.searchButton)
                self.movieList.getListItem(self.movieList.getSelectedPosition()).select(False)
                self.log("Focus switched to button panel")
            elif(self.current_focus==self.FOCUS_BUTTON_PANEL):
                _focused=None
                try:
                    _focused=self.getFocus()
                except Exception:
                    pass
                if(_focused==self.movieList):
                    self.current_focus=self.FOCUS_MOVIE_LIST
                    self.playList.reset()
                    self.setFocus(self.movieList)
                    self.movieList.getListItem(self.movieList.getSelectedPosition()).select(True)
                
        elif action.getId() in [const.ACTION_MOUSE_MOVE]:
            try:
                _focused=self.getFocus()
            except Exception:
                _focused=None
                
            if(_focused==self.movieList):
                pass
            elif(self.getFocus()==self.playList):
                pass
        elif action.getId() in [const.REMOTE_1,const.REMOTE_2,const.REMOTE_3,const.REMOTE_4,const.REMOTE_5,const.REMOTE_6,const.REMOTE_7,const.REMOTE_8,const.REMOTE_9]:
            if(self.action==const.SD_ACTION_SEARCH and self.search_object["target"]=="youtube"):
                return
            else:
                ret=self.doJumpToDialog(str(action.getId()-const.REMOTE_0))
                if(ret!=-1 and self._isPageAvailable(ret)):
                    self._jumpTo(ret)
        elif action.getId()==const.ACTION_PREV_ITEM:
            self.onControl(self.prevButton)
        elif action.getId()==const.ACTION_NEXT_ITEM:
            self.onControl(self.nextButton)
        
            
    def _getMovieForItem(self, item):
        try:
            for movie in self.movies["movies"]:
                if(str(movie["ID"])==((str)(item.getProperty("movie")))):
                    return movie
            self.log("Movie for property %s not found"%item.getProperty("movie"))
        except Exception:
            pass
        return None
    
    def _loadPlaylist(self, movie):
        self.playList.reset()
        self.playListText.setVisible(True)
        self.playListText.setLabel(const.TEXT_LOADING_PLAYLIST)
        
        try:
            self.pl=self.context.getApi().getPlaylist(movie["SOURCE_URL"], movie["PROVIDER_ID"])
            if(self.pl is None):
                raise Exception("К сожалению ресурс недоступен.")
            if(len(self.pl)==0):
                raise Exception("К сожалению ресурс недоступен.")
        except Exception,e:
            _code=""
            if hasattr(e,"code"):
                _code="%s %s"%(const.TEXT_CODE,str(e.code))
            try:
                msg=str(e)
            except Exception:
                msg="К сожалению ресурс недоступен."
            self.playListText.setLabel(const.TEXT_INTERFACE_ERROR+". %s\n%s"%(_code,msg))
            return
                    
        self.playListText.setVisible(False)
        self.log("Playlist has been generated. Size %d"%len(self.pl))
        self.log(str(self.pl))
        for item in self.pl:
            if("type" in item):
                #only m3u8 is supported
                if(item["type"] in ["M3U8","F4M"]):
                    continue
            li=xbmcgui.ListItem(label=self._encode(item["comment"]))
            self.playList.addItem(li)

        for i in range(0,self.playList.size()):
            self.playList.getListItem(i).select(False)

        self.movieList.getListItem(self.movieList.getSelectedPosition()).select(False)
        self.playList.selectItem(0)
        self.playList.getListItem(0).select(True)
        
        self._onPlaylistItemChanged()
        self.current_focus=self.FOCUS_PLAY_LIST
        self.setFocus(self.playList)
                
    def _onMovieItemChanged(self, movie):
        self.context.populateImage(movie)
        self.movieDescription.setText(self._encode(movie["DESCRIPTION"]))
        
        try:
            self.movieDescription.autoScroll(10000, 5000, 1000)
        except Exception:
            pass
        
        self.movieInfo.setText(text.getMovieInfoLong(movie))
        self.playListText.setVisible(True)
        self.playListText.setLabel(const.TEXT_PRESS_OK_FOR_LOADING_PLAYLIST)
        self.provider.setLabel(self.context.getApi().getPluginByKey(movie["PROVIDER_ID"]))
        self.movieTitle.setLabel(self._encode(movie["TITLE"]))

    def _onPlaylistItemChanged(self):
        item = self.playList.getSelectedItem()

    def _buildPanel(self, page=1):
        self._buildMovieList(page=page)
        if(self.action==const.SD_ACTION_SINGLE):
            #self.setTitleText("%d/%d"%(self.movies["page"],self.movies["pages_total"]))        
            self.setTitleText("%s"%(self._encode(self.movies["movies"][0]["TITLE"])))        
        elif(self.action==const.SD_ACTION_SEARCH):
            self.setTitleText("%s (%d/%d)"%(const.TEXT_SEARCH,self.movies["page"],self.movies["pages_total"]))        
        elif(self.action==const.SD_ACTION_GENRE):
            self.setTitleText("%s (%d/%d)"%(self._encode(self.genre["genre"]),self.movies["page"],self.movies["pages_total"]))        
        elif(self.action==const.SD_ACTION_FAV):
            self.setTitleText("%s (%d/%d)"%("Избранные",self.movies["page"],self.movies["pages_total"]))        

    def _buildPlayList(self):
        self.playList.reset()
            
    def _loadMovies(self, page=1):
        if(self.movies is None or self.movies["page"]!=page):
            if(self.action==const.SD_ACTION_SINGLE):
                pass
            elif(self.action==const.SD_ACTION_SEARCH):
                list_temp=self.context.getApi().searchMovies(title=self.search_object["title"],year=self.search_object["year"],actors=self.search_object["actors"], target=self.search_object["target"],pageToken=self.pageToken,page=page,itemsPerPage=20)
                if(len(list_temp["movies"])>0):
                    self.movies=list_temp
            elif(self.action==const.SD_ACTION_GENRE):
                list_temp=self.context.getApi().getMoviesByGenre(genreId=self.genre["id"],page=page, itemsPerPage=20)
                if(len(list_temp["movies"])>0):
                    self.movies=list_temp
            elif(self.action==const.SD_ACTION_FAV):
                list_temp=self.context.getApi().getFavorites(page=page,itemsPerPage=20)
                if(len(list_temp["movies"])>0):
                    self.movies=list_temp
                
                
        self.nextButton.setEnabled(self._isNextPageAvailable())
        #self.nextButton.setVisible(self._isNextPageAvailable())
        self.prevButton.setEnabled(self._isPrevPageAvailable())
        #self.prevButton.setVisible(self._isPrevPageAvailable())
                
        return self.movies
    
    def _clearPanel(self):
        pass
    
    def _buildMovieList(self, page=1):
        self.log("Building movie list for page %d..."%page)
        reset=False
        for movie in self._loadMovies(page=page)["movies"]:
            if(reset==False):
                self.playList.reset()
                self.movieList.reset()
                reset=True
            movie["control"]=self.image
            plugin=self.context.getApi().getPluginByKey(movie["PROVIDER_ID"])            
            li=xbmcgui.ListItem(label=self._encode(movie["TITLE"]))
            li.setProperty("movie", str(movie["ID"]))
            li.setArt({'thumb':self.context.createResourcePath('lib','plugins/%s.png'%plugin) })
            self.movieList.addItem(li)
            self.log("Movie %s added"%str(movie["ID"]))

        for i in range(0,self.movieList.size()):
            self.movieList.getListItem(i).select(False)

        self.movieList.selectItem(0)
        self.movieList.getListItem(0).select(True)
        self.current_focus=self.FOCUS_MOVIE_LIST
        self._onMovieItemChanged(self.movies["movies"][0])
        self.setFocus(self.movieList)
    
    def _fav(self):
        _list=self.doFav()
        if(_list is not None):
            #we've found someting
            self.movies=_list
            self.action=const.SD_ACTION_FAV
            self._buildPanel(page=1)
        else:
            pass     
        
    def _genre(self):
        _list,genre=self.doGenre()
        if(genre is not None):
            self.genre=genre
            self.movies=_list
            self.action=const.SD_ACTION_GENRE
            self._buildPanel(page=1)
    
    def _search(self):
        _list=self.doSearch()

        if(_list is not None):
            #we've found someting
            self.movies=_list
            self.action=const.SD_ACTION_SEARCH
            self._buildPanel(page=1)
        else:
            pass     
        
    def _tv(self):
        xbmc.executebuiltin( "ActivateWindow(busydialog)" )
        try:
            self.context.getApi().ipTvCheckCredentials()
            if(self.context.getApi().internal_iptv==False and self.context.getApi().external_iptv==False):
                dialog = xbmcgui.Dialog()
                dialog.ok(const.TEXT_SEARCH, const.TEXT_IPTV_NOT_AVAILABLE)
                del dialog
                return
            mp=IpTvPanel(context=self.context)
            if(mp.init()==True):
                xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                mp.doModal()
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
            del mp
        finally:
            xbmc.executebuiltin( "Dialog.Close(busydialog)" )                
                
        