# -*- coding: UTF-8 -*-
# Copyright 2013-2014 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cookielib
import urllib2
import re
from StringIO import StringIO
import urlparse
import xml.etree.ElementTree
import base64
import math
import struct
import urllib

try:
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.api.APIException import APIException
except ImportError:
    try:
        from resources.lib.utils.Logger import Logger
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.simplejson import loads
        from resources.lib.api.APIException import APIException
    except ImportError:
        from Logger import Logger
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from simplejson import loads
        from APIException import APIException # @UnusedImport

USE_GZIP=True    
try:
    import gzip
except ImportError:
    USE_GZIP=False
    
gst_frgmented=False

try:
    import os
    if(True or os.path.isfile("/usr/lib/gstreamer-0.10/libgstfragmented.so")==True or os.path.isfile("/usr/lib/gstreamer-1.0/libgstfragmented.so")==True):
        gst_frgmented=True
except Exception:
    pass
     
class boundFunction:
    def __init__(self, fnc, *args, **kwargs):
        self.fnc = fnc
        self.args = args
        self.kwargs = kwargs

    def __call__(self, *args, **kwargs):
        newkwargs = self.kwargs
        newkwargs.update(kwargs)
        return self.fnc(*self.args + args, **newkwargs)

class NoRedirection(urllib2.HTTPErrorProcessor):
  def http_response(self, request, response):
    return response
  https_response = http_response

class Plugin(Logger):

    USER_AGENT="Mozilla/5.0 (Windows NT 5.1; rv:33.0) Gecko/20100101 Firefox/33.0"
    ERROR_MESSAGE_INFO="К сожалению ресурс недоступен."
    ERROR_MESSAGE_FATAL="Интерфейс %d произвёл ошибку\n%s"
    
    def __init__(self, api):
        Logger.__init__(self)
        self.api=api
        self.cookies=None
        self.play_next=True
        self.cookiejar = cookielib.CookieJar()
        self._opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookiejar))
        self.log("Module "+str(self.PROVIDER_ID)+" loaded. Gzip: "+str(USE_GZIP))

    def getLocation(self, url):
        try:
            self.log("Checking Location for "+str(url))
            opener = urllib2.build_opener(NoRedirection)
            location = opener.open(url).info().getheader('Location')
            self.log("New Location "+str(location))
            if(location is not None):
                return location
            else:
                return url
        except Exception,e:
            self.log("Unable to get Location from header "+str(e))
            return url
        
    
    def isFragmentedInstalled(self):
        return gst_frgmented

    def read(self, url, options={}, data=None):
        
        response=self._getOpener(options=options).open(url, data)
        ret=None
        if(response.info().get('Content-Encoding')=="gzip" and USE_GZIP):
            f = gzip.GzipFile(fileobj=StringIO(response.read()))
            ret = f.read()
        else:
            ret = response.read()
        response.close()
        return ret
    
    def isSeekable(self):
        return True
    
    def playNext(self):
        return self.play_next

    def getPlaylist(self, url):
        try:
            return self._getPlaylist(url)
        except Exception,e:
            import traceback
            traceback.print_exc()
            raise e

    def _getOpener(self, options={}):
        self._opener.addheaders = [('User-Agent', self.USER_AGENT)]
        if(USE_GZIP):
            self._opener.addheaders.append(('Accept-encoding', 'gzip'))
        for key in options:
            self._opener.addheaders.append((key,options[key]))
        return self._opener

    def _getParameter(self, params, name, delimiter="&"):
        for q in params.split(delimiter):
            d=q.split("=")
            if(d[0]==name):
                return "=".join(d[1:])
        return None
    
    def _parse_qs(self, qs, keep_blank_values=0, strict_parsing=0):
        _dict = {}
        for name, value in urlparse.parse_qsl(qs, keep_blank_values, strict_parsing):
            _dict[name] = value
        return _dict
    
    
    def _createPlaylist(self, pl, _future=None, rememberable=False):
        ret=[]
        if("playlist" in pl):
            for item in pl["playlist"]:
                pair = {}
                if("comment" in item):
                    pair["comment"] = re.sub("<br>", " ", item["comment"])
                    pair["comment"] = re.sub("<.*?>", "", pair["comment"])
                if("file" in item):
                    pair["file"] = item["file"]
                    if(rememberable==True):
                        pair["rememberable"] = True
                        
#                    if(pair["file"][0:4]!="http" and _future !=None):
                    if(_future !=None):
                        pair["future"]=boundFunction(_future, pair["file"])
                if("comment" in pair and "file" in pair):
                    pair["seen"]=False
                    ret.append(pair)
                if("playlist" in item):
                    r=self._createPlaylist(item)
                    for item_sec in r:
                        if("comment" in item_sec):
                            if("comment" in item):
                                _c = re.sub("<br>", " ", item["comment"])
                                _c = re.sub("<.*?>", "", _c)
                                item_sec["comment"]=_c+" "+item_sec["comment"]
                        elif("comment" in item):
                            item_sec["comment"] = re.sub("<br>", " ", item["comment"])
                            item_sec["comment"] = re.sub("<.*?>", "", item_sec["comment"])
                        ret.append(item_sec)
        return ret

    def _createSingplePlaylist(self, _file, appendix="", _future=None, rememberable=False):
        ret=[]
        pair={}
        pair["comment"]=u"Фильм"+appendix
        pair["file"]=_file
        pair["seen"]=False
        if(rememberable==True):
            pair["rememberable"] = True
        if(_future !=None):
            pair["future"]=boundFunction(_future, pair["file"])
        
        ret.append(pair)
        return ret
    
    def _createFuturePlaylist(self, _comment,_file, _future):
        pair={}
        pair["comment"]=_comment
        pair["file"]=_file
        pair["seen"]=False
        pair["future"]=_future
        return pair
    
##############################################################################

    def parseManifest(self, _xml):
        try:
            from Plugins.Extensions.Seasondream.Config import cfg
        except ImportError,e:
            import traceback
            traceback.print_exc()

        root=xml.etree.ElementTree.fromstring(_xml)
        
        _media_data = {}
        _media_data["duration"] = float(root.find("{http://ns.adobe.com/f4m/1.0}duration").text)
        baseURL=root.find("{http://ns.adobe.com/f4m/1.0}baseURL")
        if(baseURL is not None):
            _media_data["baseURL"]=baseURL.text
        bitrate_list=[]
        for media in root.findall("{http://ns.adobe.com/f4m/1.0}media"):
            if("bitrate" in media.attrib):
                bitrate_list.append(int(media.attrib["bitrate"]))
        
        bitrate_list.sort()
        _des_quality="720"
        try:
            _des_quality=cfg.vkQuality.getValue()
        except Exception:
            pass
        self.log("Desired quality "+str(_des_quality))
        _bitrate={"240":0,"360":1,"480":2,"720":3}[_des_quality]
        
        _factor=len(bitrate_list)*1.0/4.0*1.0 #4 quality types
        _pos=min(len(bitrate_list)-1, (int(math.ceil(_bitrate*_factor))))
        self.log("Ceil "+str(math.ceil(_bitrate*_factor))+", factor "+str(_factor)+", take br: "+str(bitrate_list[_pos]))

        #Parse Media
        for media in root.findall("{http://ns.adobe.com/f4m/1.0}media"):
            if("bitrate" in media.attrib and int(media.attrib["bitrate"])==bitrate_list[_pos]):
                pass
            else:
                continue
            if("bitrate" in media.attrib):
                _media_data["bitrate"]=int(media.attrib["bitrate"])
            if("bootstrapInfoId" in media.attrib):
                _media_data["bootstrapInfoId"] = media.attrib["bootstrapInfoId"]
            if("url" in media.attrib):
                _media_data["url"] = media.attrib["url"]
            if("streamId" in media.attrib):
                _media_data["streamId"] = media.attrib["streamId"]
            else:
                _media_data["streamId"]="N/A"
            #TODO: select bitrate
            _media_data["metadata"] = base64.b64decode(media.find("{http://ns.adobe.com/f4m/1.0}metadata").text)
            self.log("Found media "+_media_data["streamId"]+" bitrate "+str(_media_data["bitrate"]))
            break

        for bootstrapInfo in root.findall("{http://ns.adobe.com/f4m/1.0}bootstrapInfo"):
            if("id" in bootstrapInfo.attrib and bootstrapInfo.attrib["id"]==_media_data["bootstrapInfoId"]):
                _media_data["bootstrapInfo"]=base64.b64decode(bootstrapInfo.text)
                break

        pos, boxType, _ =self.readBoxHeader(data=_media_data["bootstrapInfo"])
        if (boxType != "abst"):
            raise APIException(-2201,"abst box not found")
        
        _media_data["segTable"], _media_data["fragTable"] = self.parseBootstrapBox(data=_media_data["bootstrapInfo"], pos=pos)
        if(_media_data["segTable"] is None or _media_data["fragTable"] is None):
            raise APIException(-2202,"segTable or fragTable not found")
            
        return _media_data


    def parseMWIframe(self, url, src):
        ret=[]
        if(url is not None):
            _available_sources=self._getAvailableSources(url,src)
            self.log("Available sources "+str(_available_sources))
            
            if("manifest_m3u8" in _available_sources and gst_frgmented==True):
                pair={}
                pair["comment"]=u"Фильм [M3U8]"
                pair["file"]=_available_sources["manifest_m3u8"]
                pair["seen"]=False
                ret.append(pair)
            
            if("manifest_f4m" in _available_sources):
                manifest_url=_available_sources["manifest_f4m"]
                _media_data=self.parseManifest(self.read(manifest_url))
                pair={}
                pair["comment"]=u"Фильм "+str(_media_data["bitrate"])+"kbps [FLV]"
                pair["file"]="movie.flv"
                pair["seen"]=False
                pair["type"]="F4M"
                pair["media_data"]=_media_data
                ret.append(pair)
            

            if("manifest_m3u8" in _available_sources):
                pair={}
                pair["comment"]=u"Фильм "+str(_media_data["bitrate"])+"kbps [DVB]"
                pair["file"]="movie.ts"
                pair["seen"]=False
                pair["media_data"]=_media_data
                pair["media_url"]=_available_sources["manifest_m3u8"]
                pair["type"]="M3U8"
                ret.append(pair)
            
            return ret
        else:
            self.log("Video token not available")
        return None

    def readBoxHeader(self, data, pos=0):
        boxSize, = struct.unpack_from(">L", data, pos)
        boxType = data[pos + 4 : pos + 8]
        if boxSize == 1:
            boxSize, = struct.unpack_from(">Q", data, pos + 8)
            boxSize -= 16
            pos += 16
        else:
            boxSize -= 8
            pos += 8
        if boxSize <= 0:
            boxSize = 0
        return (pos, boxType, boxSize)

    def readInt8(self, data, pos):
        return struct.unpack_from(">B", data, pos)[0]
    
    def readByte(self, data, pos):
        return struct.unpack_from(">B", data, pos)[0]
    
    def readInt32(self, data, pos):
        return struct.unpack_from(">L", data, pos)[0]

    def readInt24(self, data, pos):
        return struct.unpack_from(">L", "\0" + data[pos:pos + 3], 0)[0]

    
    def readInt64(self, data, pos):
        a, b     = struct.unpack_from('>LL', data, pos)
        return (a << 32) | b

    def readString(self,data, pos):
        _len=0
        while (data[pos + _len] != "\0"):
            _len+=1
            
        _str=struct.unpack_from(">"+str(_len)+"s", data, pos)[0]
        pos+=_len+1
        return (_str,pos)
    
    def parseAfrtBox(self,afrt, pos):

        fragTable         = []
        version           = self.readByte(afrt, pos)
        flags             = self.readInt24(afrt, pos + 1)
        timescale         = self.readInt32(afrt, pos + 4)
        qualityEntryCount = self.readByte(afrt, pos + 8)
        pos += 9
        qualitySegmentUrlModifiers=[]
        for _ in range(0,qualityEntryCount):
            _str,pos=self.readString(afrt, pos)
            qualitySegmentUrlModifiers.append(_str)
        
        fragEntries = self.readInt32(afrt, pos)
        pos += 4
        for _ in range(fragEntries):
            firstFragment=self.readInt32(afrt, pos)
            firstFragmentTimestamp=self.readInt64(afrt, pos + 4)
            fragmentDuration=self.readInt32(afrt, pos + 12)
            discontinuityIndicator=None
            pos += 16;
            if (fragmentDuration == 0):
                discontinuityIndicator = self.readByte(afrt, pos)
                pos+=1
            fragTable.append({"firstFragment":firstFragment,"firstFragmentTimestamp":firstFragmentTimestamp,"fragmentDuration":fragmentDuration,"discontinuityIndicator":discontinuityIndicator})
            
        return fragTable;
    
    def parseAsrtBox(self,asrt, pos):
        segTable          = []
        version           = self.readByte(asrt, pos)
        flags             = self.readInt24(asrt, pos + 1)
        qualityEntryCount = self.readByte(asrt, pos + 4)
        pos += 5
        qualitySegmentUrlModifiers=[]
        for _ in range(0,qualityEntryCount):
            _str,pos=self.readString(asrt, pos)
            qualitySegmentUrlModifiers.append(_str)
        segCount = self.readInt32(asrt, pos)
        pos += 4
        for _ in range(0,segCount):
            firstSegment = self.readInt32(asrt, pos);
            fragmentsPerSegment=self.readInt32(asrt, pos + 4);
            if (fragmentsPerSegment & 0x80000000!=0):
                fragmentsPerSegment = 0;
            segTable.append({"firstSegment":firstSegment,"fragmentsPerSegment":fragmentsPerSegment})
            pos+=8
        return segTable
            
    def parseBootstrapBox(self,data, pos):
        version = self.readByte(data, pos)
        flags = self.readInt24(data, pos + 1)
        bootstrapVersion = self.readInt32(data,pos + 4)
        byte             = self.readByte(data, pos + 8)
        profile          = (byte & 0xC0) >> 6
        update = (byte & 0x10) >> 4;

        timescale           = self.readInt32(data, pos + 9)
        currentMediaTime    = self.readInt64(data, pos + 13)
        smpteTimeCodeOffset = self.readInt64(data, pos + 21)
        pos += 29;
        movieIdentifier, pos  = self.readString(data, pos)
        
        serverEntryCount = self.readByte(data, pos)
        pos+=1
        
        serverEntryTable=[]
        for i in range(0,serverEntryCount):
            set,pos=self.readString(data, pos)
            serverEntryTable.append(set)

        qualityEntryCount = self.readByte(data, pos);
        pos+=1
        qualityEntryTable=[]
        for i in range(0,qualityEntryCount):
            qet,pos=self.readString(data, pos)
            qualityEntryTable.append(qet)

        drmData,pos          = self.readString(data, pos)
        metadata,pos         = self.readString(data, pos)
        segRunTableCount     = self.readByte(data, pos)
        pos+=1
        segTable=None
        for i in range(0,segRunTableCount):
            pos, boxType, boxSize=self.readBoxHeader(data=data, pos=pos)
            if(boxType=="asrt"):
                segTable=self.parseAsrtBox(data,pos)
            pos=pos+boxSize

        fragRunTableCount = self.readByte(data, pos)
        pos+=1
        fragTable=None
        for _ in range(0,fragRunTableCount):
            pos, boxType, boxSize=self.readBoxHeader(data=data, pos=pos)
            if(boxType=="afrt"):
                fragTable=self.parseAfrtBox(data,pos)
            pos=pos+boxSize
        return segTable, fragTable           
    
    def _getAvailableSources(self, url, src):
        import re
        ret={}
        content=src.replace("\n","").replace("\r","")                          
        options={"X-Requested-With":"XMLHttpRequest","Content-Type":"application/x-www-form-urlencoded; charset=UTF-8","Referer":url}
        for item in re.compile('xhr[.]setRequestHeader[.]*[(].*?[\'|"](.[^\'|"]*?)[\'|"].*?,.*?[\'|"](.[^\'|"]*?)[\'|"][)]').findall(content):
            options[item[0]]=item[1]
            self.log("setRequestHeader("+str(item[0])+","+str(item[1])+")")

        mw_parser = MWParser()
        mw_parser.feed(src)
        mw_parser.close()
        for meta in mw_parser.meta:
            if(meta.lower()=="csrf-token"):
                options["X-CSRF-Token"]=mw_parser.meta[meta]
                self.log("$.ajaxSetup(X-CSRF-Token,"+mw_parser.meta[meta])
                
            
        for item in re.compile('eval([(].*)').findall(src):
            _eval=item;
            self.log("Eval found, decoding...")
            data=self.api.decode(self.PROVIDER_ID, "eval"+_eval+"", _type="script")
            self.log("# "+str(data))
            data=loads(data["result"], "utf-8")
            self.log("Decoded "+str(data))
            if("requestHeader" in data):
                for k in data["requestHeader"]:
                    v=data["requestHeader"][k]
                    options[k]=v
                    self.log("Added "+k+"="+v)
            
             
        for item in re.compile('{(.[^{}]*?)}').findall(content):
            if(item.find("video_token")>-1 and item.find("d_id")>-1):
                self.log("session params "+ item)
                data=dict();
                for item2 in re.compile('(.[^:]*?):(.[^,]*?)(,|$)').findall(item):
                    key=item2[0].strip()
                    value=item2[1].strip().replace("'","").replace("\"","").replace("null","")
                    if("cd"==key):
                        continue
                    data[key]=value
                data["cd"]=0
                _parsed_url=urlparse.urlparse(url)
                self.log("Options "+str(options))
                self.log("Data "+str(data))
                self.log(_parsed_url.scheme+"://"+_parsed_url.netloc+"/sessions/create_new")
                ret=loads(self.read(_parsed_url.scheme+"://"+_parsed_url.netloc+"/sessions/create_new", options=options,data=urllib.urlencode(data)),"utf-8")
                return ret
        return None  

    def _futureMW(self, url):
        self.log("Loading future object "+str(url))
        try:
            iframe_src=self.read(url)
            _ret=self.parseMWIframe(url,iframe_src)
            if(_ret is not None):
                if(self.isFragmentedInstalled()==True):
                    for _r in _ret:
                        if("type" not in _r):
                            self.log("Playing m3u8 object "+_r["file"])
                            return _r["file"].encode("utf-8")
                else:
                    for _r in _ret:
                        if("type"  in _r and _r["type"]=="F4M"):
                            self.log("Playing f4m object")
                            return _r
                    
        except Exception:
            pass
        return None

class MWParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.meta={}
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="meta"):
            name = None
            content=None
            try:
                name=[v for k, v in attrs if k == 'name'][0]
            except Exception:
                pass
            try:
                content=[v for k, v in attrs if k == 'content'][0]
            except Exception:
                pass
            if(name is not None and content is not None):
                self.meta[name]=content
                
    def handle_endtag(self, tag):
        pass
