# -*- coding: UTF-8 -*-
# Copyright 2013-2014 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
    

class Treetv(Plugin):
    PROVIDER_ID=26

    def __init__(self,api):
        Plugin.__init__(self, api)
    
    def _getPlaylist(self, url):

        content=self.read(url,options={"Referer":url})            
        parser = TreetvParser()
        parser.feed(content)
        parser.close()
        
        _ret=[]
        
        for file_id in parser.files:
            self.log("Checking file id "+str(file_id))
            pl_url="http://player.3tv.im/list/?file="+str(file_id)+"&source=1"
            try:
                content=self.read(pl_url,options={"Referer":url})
                for item in loads(content.decode('ascii', 'ignore').encode("utf-8"),"utf-8"):
                    for source in item["sources"]:
                        self.log(item["name"]+" "+source["label"])
                        pair={}
                        pair["comment"]=item["name"]+source["label"]
                        pair["file"]=source["src"]
                        pair["seen"]=False
                        _ret.append(pair)
                                        
                break
            except Exception,e:
                self.log("Unable to load playlist for "+str(pl_url) + str(e))

        if(len(_ret)==0):
            raise APIException(-2600,self.ERROR_MESSAGE_INFO)

        return _ret
        
    
class TreetvParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.data_folder=None
        self.data_folder_found=False
        self.folders=[]
        self.files=[]
        self.deleted=False
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="div"):
            self.data_folder=None
            self.data_file_id=None
            try:
                self.data_folder=[v for k, v in attrs if k == 'data-folder'][0]
            except Exception:
                pass
            try:
                self.data_file_id=[v for k, v in attrs if k == 'data-file_id'][0]
            except Exception:
                pass
            if(self.data_file_id is not None and self.data_file_id  not in self.files):
                self.files.append(self.data_file_id)
            try:
                _clazz=[v for k, v in attrs if k == 'class'][0]
                self.deleted=_clazz=="no_files" or self.deleted 
            except Exception:
                pass

        if(tag.lower()=="a" and self.data_folder is not None):
            _title=None
            try:
                _title=[v for k, v in attrs if k == 'title'][0]
            except Exception:
                pass
            if(_title is not None):
                self.folders.append({"id":self.data_folder,"title":_title})
                self.data_folder=None
                
    def handle_endtag(self, tag):
        pass    
    