# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import urllib2
import time
import math

from operator import itemgetter
from time import sleep

from threading import Thread, Condition
from ..utils.Logger import Logger
from ..constants import const
from urlparse import urlparse
from ..simplejson import loads

def buildOpener(user_agent):
    opener = urllib2.build_opener()
    opener.addheaders = [("Connection","close")]
    opener.addheaders.append(('User-Agent', user_agent))
    return opener

def getExtXStreamInfParams(params):
    ret={}
    for pa in params.split(","):
        v=pa.split("=")
        if(len(v)==2):
            ret[v[0].upper().strip()]=v[1].strip()
    return ret
 
def getExtXStreamInfForSelectedQuality(ext_x_stream_inf, quality=3):
    try:
        ext_x_stream_inf_sorted=sorted(ext_x_stream_inf, key=itemgetter("BANDWIDTH"))
        factor=len(ext_x_stream_inf)*1.0/4.0*1.0 #4 quality types
        pos=min(len(ext_x_stream_inf)-1, (int(math.ceil(quality*factor))))
        return ext_x_stream_inf_sorted[pos]
    except Exception:
        pass
    return ext_x_stream_inf[0] 

def getUrlSuffix(url, default=""):
    try:
        parsed=urlparse(url)
        if(parsed.path.rindex(".")>-1):
            return parsed.path[parsed.path.rindex(".")-len(parsed.path):] 
    except Exception:
        pass
    return default
        
def getM3UInfos(url, user_agent):
    extinf=[]
    suffix_extinf=[]
    ext_x_stream_inf=[]
    suffix_ext_x_stream_inf=[]
    content=None
    try:
        
        def getContent():
            for _ in range(20):
                try:
                    content=buildOpener(user_agent=user_agent).open(url.encode("utf-8")).read()
                    if(len(content)>0):
                        return content
                    else:
                        sleep(0.3)
                except Exception:
                    return None
            return None
        content=getContent()
        if(content is None):
            raise Exception("No content given")

        lines=content.splitlines()
        is_extinf=False
        is_ext_x_stream_inf=False
        for line in lines:
            if line.startswith('#EXTINF'):
                ext_inf=line[8:]
                is_extinf=True
            elif line.startswith('#EXT-X-STREAM-INF'):
                ext_inf=line[18:]
                is_ext_x_stream_inf=True
            elif (is_extinf==True):
                if(line.startswith("http")==False):
                    line=url[:url.rfind("/")]+"/"+line
                extinf.append({"url":line,"extinf":ext_inf})
                ext_inf=None
                is_extinf=False
            elif (is_ext_x_stream_inf==True):
                if(line.startswith("http")==False):
                    line=url[:url.rfind("/")]+"/"+line
                stream_inf_param=getExtXStreamInfParams(ext_inf)
                try:
                    bw=(int)(stream_inf_param["BANDWIDTH"])
                except Exception:
                    bw=0
                ext_x_stream_inf.append({"url":line,"extinf":ext_inf, "ext_x_stream_inf":stream_inf_param,"BANDWIDTH":bw})
                ext_inf=None
                is_ext_x_stream_inf=False
            else:
                is_extinf=False
                is_ext_x_stream_inf=False
        for extinfo in extinf:
            suffix_extinf.append(getUrlSuffix(extinfo["url"]))
        for extinfo in ext_x_stream_inf:
            suffix_ext_x_stream_inf.append(getUrlSuffix(extinfo["url"]))
    except Exception:
        pass
    
    return (extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,content) 
 
class M3U8Downloader(Thread, Logger):
    def __init__(self, url, ring_buffer, mode, user_agent=None):
        Thread.__init__(self, name="M3U8Downloader")
        Logger.__init__(self)
        self.setDaemon(True)
        self.condition=Condition()

        self.cancelled=False
        self.vod_eof=False
        self.ring_buffer=ring_buffer
        self.url=url
        self.url_orig=url
        
        self.mode=mode
        self.user_agent=user_agent
        self._last_content=None
        self.ignore_m3u8_eol=False
        self.reload_intervall=None
        self._first_sequence=None
        self.counter=0
        self._files = {}
        
        if(mode in [const.MODE_M3U8_REGULAR,const.MODE_M3U8_FILMON]):
            if(self.user_agent is None):
                self.user_agent=const.DEFAULT_USER_AGENT
        elif(mode==const.MODE_M3U8_INTERNAL):
            if(self.user_agent is None):
                self.user_agent=const.INTERNAL_USER_AGENT
        else:
            raise RuntimeError("Unknown mode %s"%str(mode))
    
    def vodEOF(self):
        pass
    
    def addFile(self, sequence, d):
        self.condition.acquire()
        try:
            if not self._first_sequence:
                self._first_sequence=sequence
                self.log("First sequence set to "+str(self._first_sequence))
            if(self._first_sequence+self.counter<=sequence):
                self._files[sequence]=d
                self.log("Added sequence "+str(sequence))
        finally:
            self.condition.notifyAll()
            self.condition.release()    

    def getFilmonUrl(self, url):
        try:
            self.log("Loading Filmon session_key...")
            _content=buildOpener(user_agent=self.user_agent).open("http://www.filmon.com/tv/api/init").read()
            fo_session_key=loads(_content,encoding="utf-8")["session_key"]
            del _content
            self.log("Filmon session_key loaded. "+fo_session_key)
            self.log("Loading channel url "+url+str()+"?session_key="+fo_session_key)
            _content=buildOpener(user_agent=self.user_agent).open(url+"?session_key="+fo_session_key).read()
            stream_array=[s for s in loads(_content,encoding="utf-8")["streams"] if s["quality"]=="high"]
            del _content
            for s in stream_array:
                self.log("Found "+s["url"])
                o=urlparse(s["url"].replace("rtmp://","http://").encode("utf-8"))
                _url= "http://"+o.netloc+o.path+s["name"]+"/playlist.m3u8?"+o.query
                self.log("Converted to "+_url)
                return _url
            self.log("No reasonable url found")
        except Exception,e:
            self.log("Unable to load Filmon source "+str(e))
        return None

    
    def run(self):

        try:
            extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,_ = getM3UInfos(self.url, user_agent=self.user_agent)  # @UnusedVariable
            if(len(extinf)==0 and len(ext_x_stream_inf)>0):
                self.log("M3U ext x stream Playlist found")
                if(len(ext_x_stream_inf)>1):
                    self.log("Found #"+str(len(ext_x_stream_inf))+". Sorting by BANDWIDTH..")
                    self.url=getExtXStreamInfForSelectedQuality(ext_x_stream_inf)["url"]
                else:
                    self.url=ext_x_stream_inf[0]["url"]
            if(self.mode==const.MODE_M3U8_FILMON):
                self.url=self.getFilmonUrl(self.url_orig)

            while self.cancelled==False and self.vod_eof==False:
                def getContent():
                    for _ in range(20):
                        if(self.cancelled==False and self.vod_eof==False):
                            pass
                        else:
                            return None
                        try:
                            content=buildOpener(user_agent=self.user_agent).open(self.url.encode("utf-8")).read()
                            if(len(content)>0):
                                return content
                            else:
                                sleep(0.3)
                        except Exception,e:
                            self.log("Caught exception while downloading "+str(self.url)+" "+str(e))
                            if(self.mode==const.MODE_M3U8_FILMON):
                                self.url=self.getFilmonUrl(self.url_orig)
                                continue
                            else:
                                return None
                    self.log("Unable to get playlist content, giving up.")
                    return None
                content=getContent()
                if(content is None):
                    break
                if self._last_content and content == self._last_content:
                    self.log("Same content found")
                    sleep(0.3)
                    continue                
                self._last_content = content
                i=0
                _lines=content.splitlines()
                _duration=0
                _is_resource_next=False
                self.target_duration=None
                for line in _lines:
                    if line.startswith('#EXT-X-MEDIA-SEQUENCE'):
                        self.media_sequence = int(line[22:])
                        i=self.media_sequence
                    elif line.startswith('#EXT-X-ENDLIST'):
                        self.log("End of list dectected. Ignore %s"%(str(self.ignore_m3u8_eol)))
                        if(self.ignore_m3u8_eol==False):
                            self.vodEOF()
                        break
                    elif line.startswith('#EXT-X-TARGETDURATION'):
                        _l=line[22:]
                        _temp=_l.split(",")
                        if(len(_temp)>1):
                            _l=_temp[0]
                        try:
                            self.target_duration = float(_l)
                            self.log("TD "+str(self.target_duration))
                        except Exception,e:
                            self.log("Unable to parse duration "+str(e))
                    elif line.startswith('#EXTINF'): #todo check extended info
                        _l=line[8:-1]
                        if(len(_l)==0):
                            _l=line[8:]
                        _temp=_l.split(",")
                        if(len(_temp)>1):
                            _l=_temp[0]
                        try:
                            _duration=float(_l)
                            self.log("D["+str(i)+"] "+str(_duration))
                        except Exception,e:
                            self.log("Unable to parse duration "+str(e))
                        _is_resource_next=True
                    elif (_is_resource_next==True):
                        if(line.startswith("http")==False):
                            line=self.url[:self.url.rfind("/")]+"/"+line
                        d=dict(file=line, sequence=i, duration=_duration)
                        self.addFile(i,d)
                        i+=1
                        _is_resource_next=False
                    else:
                        _is_resource_next=False
                if(self.target_duration is not None):
                    if(_duration>self.target_duration):
                        _duration=self.target_duration
                
                self.log("Done d:"+str(_duration))
                if(self.reload_intervall is not None):
                    sleep(self.reload_intervall)
                else:
                    sleep(_duration*0.85)
        except Exception,e:
            self.log("Caught exception "+str(e)+" m3u8 "+self.url)
            import traceback
            traceback.print_exc()
        
        self.cancel()
        self.log("Thread terminated")            

    def cancel(self):
        if(self.cancelled==False):
            self.condition.acquire()
            try:
                pass
            finally:
                self.condition.notifyAll()
                self.condition.release()        
            self.cancelled=True
            self.log("Cancelled")
                
        
    def getNextSegment(self):
        self.condition.acquire()
        try:
            while(self._first_sequence is None and self.cancelled==False):
                self.condition.wait(60)
            while((self._first_sequence+self.counter) not in self._files and self.cancelled==False):
                self.condition.wait(60)
            if(self.cancelled==True):
                _s=None
            else:
                _pointer=self._first_sequence+self.counter
                _s=self._files[_pointer]
                del self._files[_pointer]
                self.counter+=1
            return _s
        finally:
            self.condition.notifyAll()
            self.condition.release()        


class M3U8FragmentDownloader(Thread, Logger):
    def __init__(self, url, ring_buffer, mode, user_agent=None):
        Thread.__init__(self, name="M3U8FragmentDownloader")
        Logger.__init__(self)
        self.setDaemon(True)
        self.cancelled=False
        self.ring_buffer=ring_buffer
        self.url=url
        self.mode=mode
        self.user_agent=user_agent
        if(mode in [const.MODE_M3U8_REGULAR, const.MODE_M3U8_FILMON]):
            if(self.user_agent is None):
                self.user_agent=const.DEFAULT_USER_AGENT
        elif(mode==const.MODE_M3U8_INTERNAL):
            if(self.user_agent is None):
                self.user_agent=const.INTERNAL_USER_AGENT
        else:
            raise RuntimeError("Unknown mode %s"%str(mode))
            
        self.downloader=M3U8Downloader(url=self.url, ring_buffer=self.ring_buffer, mode=self.mode, user_agent=self.user_agent)
    
    def cancel(self):
        if(self.cancelled==False):
            self.cancelled=True        
            self.ring_buffer.cancel()
            self.downloader.cancel()
            self.log("Cancelled")
            
                
    def run(self):
        self.downloader.start()
        try:
            while self.cancelled==False:
                next_ts=self.downloader.getNextSegment()
                if(next_ts is None):
                    self.log("M3u8 downloader terminated")
                    break
                start=time.time()
                def createConnection():
                    for _ in range(20):
                        try:
                            response=buildOpener(user_agent=self.user_agent).open(next_ts["file"].encode("utf-8"))
                            return response
                        except Exception,e:
                            self.log("Caught exception while downloading "+str(next_ts)+" "+str(e))
                    return None
                connection=createConnection()
                if(connection is None):
                    self.log("Unable to create connection")
                    break
                length=0
                while connection is not None:            
                    av=self.ring_buffer.getAvailableForWrite()
                    if(av is None):
                        self.log("Ring buffer terminated")
                        connection.close()
                        self.cancel()       
                        break
                    chunk = connection.read(av)
                    if not chunk:
                        connection.close()
                        break
                    length+=len(chunk)
                    self.ring_buffer.put(chunk)
                el_time=(time.time()-start)
                self.speed=length*1.0/el_time*1.0 #Bps
                self.log("Done. et:"+str(round(el_time,2))+" l:"+str(length)+" bs:"+str(self.ring_buffer.getCurrentSize())+" sp:"+str(round(self.speed/(1024.0*1024.0),2)))
        except Exception,e:
            self.log("Caught exception "+str(e)+" m3u8 "+self.url)
            import traceback
            traceback.print_exc()
        
        self.cancel()
        self.log("Thread terminated")            
            
        