# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from threading import Condition
MAX_FRAGMENTSIZE=188*1400
from ..utils.Logger import Logger

class RingBuffer(Logger):
    def __init__(self):
        Logger.__init__(self)
        self.condition=Condition()
        self.fd=None
        self.data=[]
        self.curr_size=0
        self.cancelled=False
        self.completed=False
        self.SIZE=10*1024*1024 #10MB
    
    def getCurrentSize(self):
        return self.curr_size
    
    def cancel(self):
        if(self.cancelled==False):
            self.condition.acquire()
            try:
                self.cancelled=True
                if(self.fd is not None):
                    self.fd.close()
                    self.fd=None
                del self.data[:]
            finally:
                self.condition.notifyAll()
                self.condition.release()
            self.log("Cancelled")
            
    def put(self, d):
        self.condition.acquire()
        try:
            if(self.cancelled==True):
                return
            self.curr_size+=len(d)
            if(self.fd is not None):
                self.fd.write(d)
            self.data.append(d)
        finally:
            self.condition.notifyAll()
            self.condition.release()

    def getAvailableForWrite(self):
        self.condition.acquire()
        try:
            while(self.curr_size>=self.SIZE and self.cancelled==False):
                #self.log("Buffer is full. Waiting for consumer... ")
                self.condition.wait(60)
            _l=self.SIZE-self.curr_size
            _l=min(MAX_FRAGMENTSIZE,_l)
            if(self.cancelled==True):
                _l=None
        finally:
            self.condition.notifyAll()
            self.condition.release()
        return _l
    
    def get(self):
        self.condition.acquire()
        try:
            while(self.curr_size<1 and self.cancelled==False and self.completed==False):
                #self.log("Buffer is empty. Waiting for producer...")
                self.condition.wait(60)
            if(self.completed==True and self.curr_size<1):
                return None
            if(self.cancelled==True):
                return None
            _d=self.data[0]
            del self.data[0]
            self.curr_size-=len(_d)
        finally:
            self.condition.notifyAll()
            self.condition.release()
        return _d            