# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import socket
import urllib2
import urllib
import select

from urlparse import urlparse,parse_qs
from threading import Thread
from ..utils.Logger import Logger
from ..constants import const

class Server(Thread, Logger):
    
    def __init__(self, ring_buffer, mode=const.MODE_M3U8_INTERNAL, url=None):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.settimeout(None)        
        self.cancelled=False
        self.ring_buffer=ring_buffer
        self.mode=mode
        self.url=url
        
    def init(self):
        try:
            self.s.bind(("", 0))
            self.s.listen(10)
            return (int)(self.s.getsockname()[1])
        except socket.error as msg:
            self.log("Unable to bind socket. Already in use? "+str(msg[1]))
            return -1
        
    def cancel(self):
        if(self.cancelled==False):
            try:
                self.s.close()
            except Exception,e:
                self.log("Unable to close socket "+str(e))
            self.ring_buffer.cancel()
            self.cancelled=True

    def _parserHeader(self, header):
        lines=[r.strip() for r in header.split("\n")]
        headline=lines[0]
        action = headline.split(" ")
        method=action[0]
        path=action[1]
        params=dict()
        header_params=dict()
        for _item in parse_qs(urlparse("http://localhost"+path).query, keep_blank_values=True).items():
            params[_item[0]]=urllib.unquote(_item[1][0]).decode("utf-8")                    
        for _i in range(1,len(lines)):
            _pos=lines[_i].find(":")
            if(_pos>-1):
                header_params[lines[_i][:_pos].strip()]=lines[_i][_pos+1:].strip()
        return (method, path, params, header_params)
    
    def _sendHeader(self, conn, status_code=200, status_text="OK",  params={}):
        
        header=[]
        header.append('HTTP/1.1 %d %s'%(status_code,status_text))
        header.append('Server: Seasondream')
        header.append('Content-Type: application/octet-stream')
        header.append('Cache-Control: no-cache')
        header.append('Connection: keep-alive')
        header.append('Accept-Ranges: bytes')
        for _key in params.keys():
            header.append("%s: %s"%(_key.strip(),params[_key].strip()))
        
        conn.sendall('\r\n'.join(header)+"\r\n\r\n")
        self.log("Response header has been sent")
            
    def _run_proxy(self):
        bytes_sent_total=0
        while True:
            conn, addr = self.s.accept()  # @UnusedVariable
            request=conn.recv(1024)
            self.log("Client accepted. "+request) #Free buffer
            

            lines=[r.strip() for r in request.split("\n")]
            headline=lines[0]
            action = headline.split(" ")
            method=action[0]
            path=action[1]
            if(method=="HEAD"):
                header="HTTP/1.1 200 OK\r\nConnection:close\r\nServer: Seasondream\r\nContent-Type: application/octet-stream\r\n\r\n"
                conn.sendall(header)
                conn.close()
                
                conn, addr = self.s.accept()  # @UnusedVariable
                request=conn.recv(1024)
                self.log("Client accepted. "+request) #Free buffer
                lines=[r.strip() for r in request.split("\n")]
                headline=lines[0]
                action = headline.split(" ")
                method=action[0]
                path=action[1]
            
            try:
                _, _, _, header_params=self._parserHeader(request)
                code=(200,'OK')
                opener = urllib2.build_opener()
                opener.addheaders = [('User-Agent', const.DEFAULT_USER_AGENT)]
                if("Range" in header_params):
                    self.log("Client is asking for range "+header_params["Range"])
                    import re
                    _range=re.compile("bytes=([0-9]+?)[-]([0-9]*?)$").match(header_params["Range"])
                    bytes_sent_total= (int)(_range.group(1))
                    code=(206,'Partial content')
                    opener.addheaders.append(('Range',header_params["Range"]))

                content_length=None
                self.log("Using opener headers "+str(opener.addheaders))
                response = opener.open(self.url)
                header=[]
                self.log(response.headers)
                header.append("HTTP/1.1 %d %s"%(code))
                for _h in [_h for _h in ["Content-Length","Accept-Ranges","Content-Range","Content-Type","Connection"] if response.headers.get(_h)]:
                    header.append("%s: %s"%(_h,response.headers.get(_h)))
                    if(_h=="Content-Length"):
                        content_length=(int)(response.headers.get(_h))
                    if(_h=="Content-Range"):
                        _range=re.compile("bytes ([0-9]+?)[-]([0-9]+?)/([0-9]+?)$").match(response.headers.get(_h))
                        content_length=(int)(_range.group(3))

                header.append("\r\n")
                conn.sendall("\r\n".join(header))
                _counter_failed=0
                while _counter_failed<20:
                    try:
                        block=response.read(1024*512)
                        if(len(block)==0 and self.mode==const.MODE_PROXY_UDPXY):
                            raise Exception("Connection terminated. Bytes sent:%d"%(bytes_sent_total))
                        if(len(block)==0 and content_length is not None and bytes_sent_total<content_length):
                            raise Exception("Connection terminated. Bytes sent:%d, Total:%d"%(bytes_sent_total,content_length))
                    except Exception,e:
                        _counter_failed+=1
                        self.log("Unable to read from source "+str(e))
                        response.close()
                        opener = urllib2.build_opener()
                        opener.addheaders = [('User-Agent', const.DEFAULT_USER_AGENT)]
                        if(content_length is not None):
                            opener.addheaders.append (('Range','bytes=%d-%d'%(bytes_sent_total,content_length)))
                        self.log("Headers "+str(opener.addheaders))
                        response = opener.open(self.url)
                        if(response.getcode()!=206 and  self.mode!=const.MODE_PROXY_UDPXY):
                            self.log("Wrong response code %d, expected 206"%(response.getcode()))
                            break #terminate connection
                        else:
                            continue
                    if len(block)==0:
                        break #eof?
                    conn.sendall(block)
                    bytes_sent_total+=len(block)
                    if(content_length is not None and content_length==bytes_sent_total):
                        self.log("Processed")
                        break #terminate connection
                break
            except Exception,e:
                self.log("Caught exception "+str(e))
                import traceback
                traceback.print_exc()
                
                if(self.cancelled==True):
                    break
                readable, _, _ = select.select([self.s], [self.s], [], 1)
                if(len(readable)>0):
                    self.log("New client data available")
                    continue
                else:
                    self.log("No further client data available, close connection")
                    break
        self.cancel()
        self.log("Thread terminated")     
            
    def _run_normal(self):
        conn, addr = self.s.accept()  # @UnusedVariable
        
        request=conn.recv(1024)
        self.log("Client accepted. "+request) #Free buffer

        lines=[r.strip() for r in request.split("\n")]
        headline=lines[0]
        action = headline.split(" ")
        method=action[0]
        path=action[1]
        if(method=="HEAD"):
            header="HTTP/1.1 200 OK\r\nConnection:close\r\nServer: Seasondream\r\nContent-Type: application/octet-stream\r\n\r\n"
            conn.sendall(header)
            conn.close()
            
            conn, addr = self.s.accept()  # @UnusedVariable
            request=conn.recv(1024)
            self.log("Client accepted. "+request) #Free buffer
            lines=[r.strip() for r in request.split("\n")]
            headline=lines[0]
            action = headline.split(" ")
            method=action[0]
            path=action[1]
                
        if(method=="GET"):
            header="HTTP/1.1 200 OK\r\nConnection:close\r\nServer: Seasondream\r\nContent-Type: application/octet-stream\r\n\r\n"
            conn.sendall(header)
            self.log("Response header has been sent")
            try:
                while self.cancelled==False:
                    _data=self.ring_buffer.get()
                    if(_data is None):
                        break
                    conn.sendall(_data)
            except Exception,e:
                self.log("Caught exception "+str(e))
        else:
            header="HTTP/1.1 400 Bad request\r\nConnection:close\r\nServer: Seasondream\r\nContent-Type: application/octet-stream\r\n\r\n"
            conn.sendall(header)
            conn.close()
        self.cancel()
        self.log("Thread terminated")        
    
    def run(self):
        if(self.mode in [const.MODE_M3U8_INTERNAL,const.MODE_M3U8_REGULAR,const.MODE_M3U8_FILMON, const.MODE_XTREAM_CODES]):
            self._run_normal()
        elif(self.mode in [const.MODE_PROXY,const.MODE_PROXY_UDPXY]):
            self._run_proxy()
        else:
            self.log("Unknown mode "+str(self.mode))  
