# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .RingBuffer import RingBuffer
from .Server import Server
from .M3U8Processor import M3U8FragmentDownloader
from .XtreamProcessor import XtreamFragmentDownloader
from ..constants import const


class Streamer:
    
    def __init__(self, url, mode, context=None):
        self.mode=mode
        self.url=url
        self.ring_buffer=RingBuffer()
        self.server=Server(ring_buffer=self.ring_buffer, mode=self.mode,url=self.url)
        self.downloader=None
        self.port=None
        self.context=context
    
    def init(self):
        if(self.mode in[const.MODE_M3U8_INTERNAL,const.MODE_M3U8_REGULAR,const.MODE_M3U8_FILMON]):
            self.port=self.server.init()
            if(self.port!=-1):
                self.downloader=M3U8FragmentDownloader(url=self.url, ring_buffer=self.ring_buffer, mode=self.mode)
                return self.port
        elif(self.mode in[const.MODE_PROXY,const.MODE_PROXY_UDPXY]):
            self.port=self.server.init()
            if(self.port!=-1):
                self.downloader=None
                return self.port
        elif(self.mode in[const.MODE_XTREAM_CODES]):
            self.port=self.server.init()
            if(self.port!=-1):
                self.downloader=XtreamFragmentDownloader(url=self.url, ring_buffer=self.ring_buffer, mode=self.mode, context=self.context)
                return self.port
        return -1
    
    def cancel(self):
        self.server.cancel()
        self.ring_buffer.cancel()
                
    def start(self):
        if(self.downloader is not None):
            self.downloader.start()
        self.server.start()
