# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import urllib2
import time
import os
import random

from time import sleep

from threading import Thread, Condition
from ..utils.Logger import Logger
from ..constants import const

def buildOpener(user_agent):
    opener = urllib2.build_opener()
    opener.addheaders = [("Connection","close")]
    opener.addheaders.append(('User-Agent', user_agent))
    return opener

 
 
class XtreamFragmentDownloader(Thread, Logger):
    def __init__(self, url, ring_buffer, mode, user_agent=None, context=None):
        Thread.__init__(self, name="XtreamFragmentDownloader")
        Logger.__init__(self)
        self.setDaemon(True)
        self.cancelled=False
        self.ring_buffer=ring_buffer
        self.url=url
        self.last_master=None
        self.mode=mode
        self.user_agent=user_agent
        if(mode in [const.MODE_XTREAM_CODES]):
            if(self.user_agent is None):
                self.user_agent=const.DEFAULT_USER_AGENT
        else:
            raise RuntimeError("Unknown mode %s"%str(mode))

        self.downloader=XtreamDownloader(url=self.url, ring_buffer=self.ring_buffer, mode=self.mode, user_agent=self.user_agent, context=context)
    
    def cancel(self):
        if(self.cancelled==False):
            self.cancelled=True
            if(self.downloader is not None):
                self.downloader.cancel()
            self.ring_buffer.cancel()
            self.log("XtreamFragmentDownloader cancelled")
            
    def run(self):
        try:
            import mmap
            self.downloader.start()
            while self.cancelled==False:
                dat=self.downloader.getFromQueue()
                if(dat is None):
                    break
                master, segment, last_data= dat
                _file=self.downloader.file_pattern%(master, segment)
                self.log("Got %d|%d"%(master,segment))
                if(self.last_master is None):
                    self.last_master=master
                    
                def writeToRingBuffer(_from=0, _file=None):
                    length=0
                    with open(_file,"r+b") as f:
                        f.seek(_from)
                        while True:
                            av=self.ring_buffer.getAvailableForWrite()
                            if(av is None):
                                self.cancel()
                                break
                            chunk = f.read(av)
                            if not chunk:
                                break
                            length+=len(chunk)
                            self.ring_buffer.put(chunk)
                        
                                                    
                        f.seek(_from)
                        self.downloader.condition_total.acquire()
                        try:
                            with open(self.downloader.file_total, 'a+b') as outFile:
                                for chunk in iter(lambda: f.read(32*188), ''):
                                    outFile.write(chunk)
                            if(os.path.getsize(self.downloader.file_total)>188*111550): #more than 20MB
                                with open(self.downloader.file_total, "r+b") as outFile:
                                    mm = mmap.mmap(outFile.fileno(), 0)
                                    _size=mm.size()
                                    _block=188*27887 # 5MB cut
                                    mm.move(0,_block, _size-_block)
                                    mm.resize(_size-_block)
                                    mm.flush()
                                    mm.close()
                                self.log("Total file cut %d"%(os.path.getsize(self.downloader.file_total)))                                                                   
                        finally:
                            self.downloader.condition_total.notifyAll()    
                            self.downloader.condition_total.release()                            
                if(self.last_master!=master):
                    #new master segment
                    with open(_file, "r+b") as f:
                        mm = mmap.mmap(f.fileno(), 0)
                        _pos=mm.find(self.last_data)
                        mm.close()
                    if(_pos>-1):
                        #found
                        self.log("Sync found")
                        writeToRingBuffer(_from=_pos+len(self.last_data),_file=_file)#write
                        self.last_data=last_data
                        self.last_master=master                    
                        
                    else:
                        #not found, look into the total file
                        self.log("Sync not found")
                        self.downloader.condition_total.acquire()
                        try:
                            with open(self.downloader.file_total, "r+b") as f:
                                mm = mmap.mmap(f.fileno(), 0)
                                _pos=mm.find(last_data)
                                mm.close()
                        finally:
                            self.downloader.condition_total.notifyAll()    
                            self.downloader.condition_total.release()
                        if(_pos>-1):
                            #found in buffer
                            self.log("Sync in total file %d"%(_pos))
                            pass
                        else:
                            # not in buffer, write
                            self.log("Sync not in total file, write")
                            writeToRingBuffer(_from=0,_file=_file)#write
                            self.last_master=master                    
                            self.last_data=last_data
                                                    
                else:
                    writeToRingBuffer(_from=0,_file=_file)#write
                    self.last_data=last_data
                    self.last_master=master                    
                os.remove(_file)                

        except Exception, e:
            self.log("Caught exception "+str(e))
            import traceback
            traceback.print_exc()
        self.cancel()
        self.downloader.condition_total.acquire()
        if(os.path.isfile(self.downloader.file_total)):
            os.remove(self.downloader.file_total)
        self.downloader.condition_total.notifyAll()    
        self.downloader.condition_total.release()
        
        self.log("Thread terminated")     
        
        
        
        
class XtreamDownloader(Thread, Logger):
    def __init__(self, url, ring_buffer, mode, user_agent=None, context=None):
        Thread.__init__(self, name="XtreamFragmentDownloader")
        Logger.__init__(self)
        self.setDaemon(True)
        self.cancelled=False
        self.context=context
        
        self.uuid=str(random.random()*10000000L)
        self.url=url
        
        self.file_pattern=os.path.join(self.context.getXtreamPath(),"ts_"+self.uuid+"_%d_%d.ts")
        self.file_total=os.path.join(self.context.getXtreamPath(),"ts_"+self.uuid+"_total.ts")

        self.opener = buildOpener(user_agent=user_agent)

        self.condition_total=Condition()
        self.condition=Condition()
        self.segment=0
        self.master=1
        self.queue=[]
    
    def cancel(self):
        self.cancelled=True
        self.condition.acquire()
        try:
            self.cancelled=True
        finally:
            self.condition.notifyAll()    
            self.condition.release()
        
    def _getSegment(self):
        self.segment+=1
        return self.segment
            
    def _nextMaster(self):
        self.master+=1
        self.segment=0
    
    def _getMaster(self):
        return self.master

    def _addToQueue(self, master,segment, last_data):
        self.condition.acquire()
        try:
            self.queue.append((master,segment,last_data))
        finally:
            self.condition.notifyAll()    
            self.condition.release()
            
    def getFromQueue(self):
        self.condition.acquire()
        try:
            while(len(self.queue)<1 and self.cancelled==False):
                self.condition.wait(60)
            if(self.cancelled==False and len(self.queue)>0):
                _s= self.queue[0]
                del self.queue[0]
                return _s
            else:
                return None
        finally:
            self.condition.notifyAll()    
            self.condition.release()

    def _cleanup(self):
        try:
            for _file in os.listdir(os.path.join(self.context.getXtreamPath())):
                if(_file.startswith("ts_"+self.uuid)):
                    file_path = os.path.join(self.context.getXtreamPath(), _file)
                    try:
                        os.remove(file_path)
                    except Exception:
                        pass
        except Exception:
            pass
            
    def run(self):
        try:
            import mmap
            last_response=None
            last_block=None
            counter_failed=0
            while self.cancelled==False:
                def connect(link):
                    while self.cancelled==False:
                        try:
                            return self.opener.open(link,timeout=5)
                        except Exception,e:
                            self.log( str(e)+" retry...")
                    return None
                response=connect(self.url)
                total_length=0
                _start=time.time()
                
                while self.cancelled==False:
                    try:
                        block = response.read(188*20)
                    except:
                        break
                    if block is None:
                        break #eof?
                    if len(block)<188:
                        break #eof?
                    # Access total file
                    def getRemainingBufferSize(block):
                        self.condition_total.acquire()
                        try:
                            if(os.path.isfile(self.file_total)):
                                with open(self.file_total,"r+b") as ft:
                                    mm = mmap.mmap(ft.fileno(), 0)
                                    _pos=mm.rfind(block[:min(188*5, len(block))])
                                    mm.close()
                                if(_pos>-1):
                                    return os.path.getsize(self.file_total)-_pos
                                else:
                                    return -1
                            else:
                                return -1
                        except Exception:
                            return -1
                        finally:
                            self.condition_total.notifyAll()    
                            self.condition_total.release()
                    remaining_buffer_size=getRemainingBufferSize(block)
                    self.log("Remaining bs:%d"%(remaining_buffer_size))
                        #block not in buffer
                    force_no_skip=False
                    if(remaining_buffer_size<0):
                        counter_failed+=1
                        if(counter_failed < 6):
                            #sleep(1)
                            self.log("Not in buffer. counter %d"%(counter_failed))
                            #break
                        if(last_response is not None):
                            block=last_block
                            response.close()
                            response=last_response
                            last_response=None
                            last_block=None
                            self.log("Switch to the previuos connection")
                            remaining_buffer_size=getRemainingBufferSize(block)
                            self.log("Remaining bs:%d"%(remaining_buffer_size))
                            force_no_skip=True
                    
                    
                    if(remaining_buffer_size>-1):
                        if(last_response is not None):
                            last_response.close()
                        counter_failed=0
                        last_response=response
                        last_block=block
                        #block in buffer
                        if(force_no_skip==False and remaining_buffer_size>3*1024*1024):
                            self.log("Full skip")
                            sleep(1)
                            break # too much, new download request
                        else:
                            def skip(size):
                                length=0
                                if(size<=0):
                                    return 0
                                while self.cancelled==False:
                                    try:
                                        _block = response.read(min(188*4577, (size-length)))# 256KB
                                    except Exception:
                                        break
                                    if _block is None or len(_block)==0:
                                        break #eof?
                                    length+=len(_block)
                                    del _block
                                    if(length>=size):
                                        break
                                return length
                            # skipping
                            to_skip=max(0,remaining_buffer_size-len(block)-10*188)
                            self.log("Skipping %d bytes"%(to_skip))
                            bytes_skipped=skip(to_skip)
                            if(bytes_skipped<to_skip):
                                self.log("Unable to skip %d bytes, skipped %d bytes only, close connection"%(to_skip,bytes_skipped))
                                last_response.close()                                
                                last_response=None
                                break #connection broke
                            self.log("Skipped %d bytes"%(bytes_skipped))
                            block=None                       
                    else:
                        #block not in buffer
                        pass
                    
                    # now we are ready to read/write
                    def writeBlock(block):
                        length=0
                        if(block is not None):
                            segment_counter=self._getSegment()
                            master_counter=self._getMaster()
                            _file=self.file_pattern%(master_counter,segment_counter)                            
                            with open(_file,"wb") as ft:
                                ft.write(block)
                            length+=len(block)
                            #add to queue
                            self._addToQueue(master_counter,segment_counter, block[-2*188:])
                            #self.log("Block %d wrote"%(length))
                        return length
                    
                    total_length+=writeBlock(block)                                        
                    while self.cancelled==False:
                        try:
                            block = response.read(188*697)
                        except Exception:
                            break
                        if block is None:
                            break #eof?
                        if len(block)<188:
                            break #eof?
                        total_length+=writeBlock(block)
                
                if(total_length>0):
                    self._nextMaster()

                if(response is not None):   
                    #response.close()
                    pass
                 
                _duration=time.time()-_start
                self.log("Done. d:%d l:%d"%(_duration,total_length))
                 
       
        except Exception, e:
            self.log("Caught exception "+str(e))
            import traceback
            traceback.print_exc()
        self.cancel()
        self._cleanup()
        self.log("Thread terminated")        