# -*- coding: UTF-8 -*-
# Copyright 2013-2016 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import urllib2
import time
from threading import Thread


from ..utils.Logger import Logger
from ..constants import const
from threading import Condition

class ImageLoader(Logger):
    
    def __init__(self, imagePath, cacheSize):
        self.imagePath=imagePath
        self.condition=Condition()
        self.MAX_CACHE_SIZE=cacheSize        
        self.MAX_DELETE_IF_FULL=(int)(self.MAX_CACHE_SIZE*0.1) # 10% from cache size
        self.filesTotal=len(os.listdir(self.imagePath))
        self.log("Max #files: %d, in use: %d"%(self.MAX_CACHE_SIZE,self.filesTotal))

    def _encode(self, text):
        try:
            return text.encode("utf-8")
        except Exception:
            return text
        
    def _getImageUrl(self, movie):
        if("IMG_EXT" in movie and movie["IMG_EXT"] is not None):
            return self._encode(movie["IMG_EXT"])
        return None
    
    def _getDestFile(self, movie):
        return os.path.join(self.imagePath,"%s.tmp"%str(movie["ID"]))
    
    def _downloadImage(self, url, dst):
        opener=urllib2.build_opener()#urllib2.ProxyHandler({'http': '172.16.107.110'}))
        opener.addheaders = [('User-Agent', const.DEFAULT_USER_AGENT)]
        fp=opener.open(url)
        bs = 1024*256
        with open(dst, "wb") as tfp:
            while 1:
                block = fp.read(bs)
                if block == "":
                    break
                tfp.write(block)
        fp.close()
    
    def _process(self, url, dst, movie):
        self.condition.acquire()
        try:
            if(self.filesTotal+1>self.MAX_CACHE_SIZE):
                #cache full
                _start=time.time()
                file_array=[]
                for name in os.listdir(self.imagePath):
                    _file_to_check=os.path.join(self.imagePath, name)
                    file_array.append((os.stat(_file_to_check).st_atime,name))
                def getKey(item):
                    return item[0]
                file_array=sorted(file_array, key=getKey)
                c=0
                for _,name in file_array:
                    os.remove(os.path.join(self.imagePath, name))
                    self.filesTotal-=1
                    c+=1
                    if(c>=self.MAX_DELETE_IF_FULL):
                        break
                _duration=time.time()-_start
                self.log("Cache cleaned files total: %d, elapsed time: %d"%(self.filesTotal,_duration))                
        finally:
            self.condition.notifyAll()
            self.condition.release()
        
        try:
            self._downloadImage(url, dst)
            movie["control"].setImage(dst, useCache=True)
        except Exception, e:
            movie["control"].setImage('', useCache=True)
            self.log("Unable to download/set image %s"%str(e))
        
        
    def populateImage(self, movie):
        if(movie is not None):
            url=self._getImageUrl(movie)
            dst=self._getDestFile(movie)
            #self.log(url)
            #self.log(dst)
            if(os.path.isfile(dst)):
                try:
                    movie["control"].setImage(dst, useCache=True)
                except Exception,e:
                    self.log("Unable to set control %s"%str(e))
            else:
                t=Thread(target=self._process, args=(url, dst,movie,))
                t.setDaemon(True)
                t.start()            
        
        
        